@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 1991, 1992 Paul Kranenburg <pk@cs.few.eur.nl>
  * Copyright (c) 1993 Branko Lankester <branko@hacktic.nl>
- * Copyright (c) 1993, 1994 Rick Sladkey <jrs@world.std.com>
+ * Copyright (c) 1993, 1994, 1995, 1996 Rick Sladkey <jrs@world.std.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -12,11 +12,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by Paul Kranenburg,
- *      Branko Lankester and Rick Sladkey.
- * 4. The name of the author may not be used to endorse or promote products
+ * 3. The name of the author may not be used to endorse or promote products
  *    derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
@@ -30,7 +26,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	defs.h,v 2.25 1994/03/21 05:45:25 jrs Exp
+ *	$Id: defs.h,v 2.38 1996/05/21 03:20:55 jrs Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -52,7 +48,7 @@
 #define DEFAULT_ACOLUMN	40	/* default alignment column for results */
 #endif
 #ifndef MAX_ARGS
-#define MAX_ARGS	8	/* maximum number of args to a syscall */
+#define MAX_ARGS	32	/* maximum number of args to a syscall */
 #endif
 #ifndef DEFAULT_SORTBY
 #define DEFAULT_SORTBY "time"	/* default sorting method for call profiling */
@@ -80,7 +76,7 @@
 #undef ptrace
 #ifdef __STDC__
 #ifdef LINUX
-extern int ptrace(int, int, char *, int);
+extern long ptrace(int, int, char *, long);
 #else /* !LINUX */
 extern int ptrace(int, int, char *, int, ...);
 #endif /* !LINUX */
@@ -92,17 +88,34 @@ extern int ptrace();
 #ifdef LINUX
 #define	PTRACE_PEEKUSER	PTRACE_PEEKUSR
 #define	PTRACE_POKEUSER	PTRACE_POKEUSR
+#ifdef ALPHA
+#define REG_R0 0
+#define REG_A0 16
+#define REG_A3 19
+#define REG_FP 30
+#define REG_PC 64
+#endif /* ALPHA */
+#endif /* LINUX */
+
+#define SUPPORTED_PERSONALITIES 1
+#define DEFAULT_PERSONALITY 0
+
+#ifdef LINUX
+#ifdef SPARC
+#undef SUPPORTED_PERSONALITIES
+#define SUPPORTED_PERSONALITIES 3
+#endif /* SPARC */
 #endif /* LINUX */
 
 /* Trace Control Block */
 struct tcb {
 	short flags;		/* See below for TCB_ values */
 	int pid;		/* Process Id of this entry */
-	unsigned int scno;	/* System call number */
+	long scno;		/* System call number */
 	int u_nargs;		/* System call arguments */
-	int u_arg[MAX_ARGS];	/* System call arguments */
+	long u_arg[MAX_ARGS];	/* System call arguments */
 	int u_error;		/* Error code */
-	int u_rval;		/* (first) return value */
+	long u_rval;		/* (first) return value */
 	FILE *outf;		/* Output file for this process */
 	char *auxstr;		/* Auxiliary info from syscall (see RVAL_STR) */
 	struct timeval stime;	/* System time usage as of last process wait */
@@ -113,8 +126,8 @@ struct tcb {
 	int nchildren;		/* # of traced children */
 	int waitpid;		/* pid(s) this process is waiting for */
 				/* (1st arg of wait4()) */
-	int baddr;		/* `Breakpoint' address */
-	int inst[2];		/* Instructions on above */
+	long baddr;		/* `Breakpoint' address */
+	long inst[2];		/* Instructions on above */
 	int pfd;		/* proc file descriptor */
 #ifdef SVR4
 	prstatus_t status;	/* procfs status structure */
@@ -133,6 +146,11 @@ struct tcb {
 #define TCB_SIGTRAPPED	00200	/* Process wanted to block SIGTRAP */
 #define TCB_FOLLOWFORK	00400	/* Process should have forks followed */
 #define TCB_REPRINT	01000	/* We should reprint this syscall on exit */
+#ifdef LINUX
+#ifdef ALPHA
+#define TCB_WAITEXECVE	02000	/* ignore SIGTRAP after exceve */
+#endif /* ALPHA */
+#endif /* LINUX */
 
 /* qualifier flags */
 #define QUAL_TRACE	0001	/* this system call should be traced */
@@ -177,20 +195,14 @@ struct xlat {
 /* get offset of member within a user struct */
 #define uoff(member)	offsetof(struct user, member)
 
-struct sysent {
-	int	nargs;
-	int	(*sys_func)();
-	char	*sys_name;
-};
+#define TRACE_FILE	001	/* Trace file-related syscalls. */
+#define TRACE_IPC	002	/* Trace IPC-related syscalls. */
+#define TRACE_NETWORK	004	/* Trace network-related syscalls. */
+#define TRACE_PROCESS	010	/* Trace process-related syscalls. */
+#define TRACE_SIGNAL	020	/* Trace signal-related syscalls. */
 
-extern struct sysent sysent[];
-extern int nsyscalls;
 extern struct tcb tcbtab[];
 extern int qual_flags[];
-extern char *signalent[];
-extern int nsignals;
-extern char *errnoent[];
-extern int nerrnos;
 extern int debug, followfork, followvfork;
 extern int rflag, tflag, dtime, cflag, xflag, qflag;
 extern int acolumn;
@@ -205,6 +217,7 @@ extern struct tcb *tcp_last;
 #define P(args) ()
 #endif
 
+extern int set_personality P((int personality));
 extern char *xlookup P((struct xlat *, int));
 extern struct tcb *alloctcb P((int));
 extern void droptcb P((struct tcb *));
@@ -213,22 +226,23 @@ extern void set_sortby P((char *));
 extern void set_overhead P((int));
 extern void qualify P((char *));
 extern void newoutf P((struct tcb *));
-extern int syscall P((struct tcb *));
+extern int trace_syscall P((struct tcb *));
 extern void printxval P((struct xlat *, int, char *));
 extern int printargs P((struct tcb *));
 extern int addflags P((struct xlat *, int));
 extern int printflags P((struct xlat *, int));
-extern int umoven P((struct tcb *, int, int, char *));
-extern int umovestr P((struct tcb *, int, int, char *));
-extern int upeek P((int, int, int *));
-extern void dumpstr P((struct tcb *, int, int));
-extern void printstr P((struct tcb *, int, int));
-extern void printnum P((struct tcb *, int, char *));
-extern void printpath P((struct tcb *, int));
-extern void printpathn P((struct tcb *, int, int));
-extern void printtv P((struct tcb *, int));
-extern void printsock P((struct tcb *, int));
-extern void printrusage P((struct tcb *, int));
+extern int umoven P((struct tcb *, long, int, char *));
+extern int umovestr P((struct tcb *, long, int, char *));
+extern int upeek P((int, long, long *));
+extern void dumpstr P((struct tcb *, long, int));
+extern void string_quote P((char *str));
+extern void printstr P((struct tcb *, long, int));
+extern void printnum P((struct tcb *, long, char *));
+extern void printpath P((struct tcb *, long));
+extern void printpathn P((struct tcb *, long, int));
+extern void printtv P((struct tcb *, long));
+extern void printsock P((struct tcb *, long));
+extern void printrusage P((struct tcb *, long));
 extern int clearbpt P((struct tcb *));
 extern int setbpt P((struct tcb *));
 extern int sigishandled P((struct tcb *, int));
@@ -238,11 +252,17 @@ extern void printleader P((struct tcb *));
 extern void printtrailer P((struct tcb *));
 extern void tabto P((int));
 extern void call_summary P((FILE *));
+extern void fake_execve P((struct tcb *, char *, char *[], char *[]));
 
-extern char *ioctl_lookup P((int));
-extern int ioctl_decode P((struct tcb *, int, int));
-extern int term_ioctl P((struct tcb *, int, int));
-extern int sock_ioctl P((struct tcb *, int, int));
+extern int internal_fork P((struct tcb *));
+extern int internal_exec P((struct tcb *));
+extern int internal_wait P((struct tcb *));
+extern int internal_exit P((struct tcb *));
+
+extern char *ioctl_lookup P((long));
+extern int ioctl_decode P((struct tcb *, long, long));
+extern int term_ioctl P((struct tcb *, long, long));
+extern int sock_ioctl P((struct tcb *, long, long));
 extern int proc_ioctl P((struct tcb *, int, int));
 extern int stream_ioctl P((struct tcb *, int, int));
 
@@ -259,7 +279,7 @@ extern void tv_div P((struct timeval *, struct timeval *, int));
 extern int fixvfork P((struct tcb *));
 #endif
 #ifndef LINUX
-extern int getrval2 P((struct tcb *));
+extern long getrval2 P((struct tcb *));
 #endif
 #ifdef SVR4
 extern int proc_open P((struct tcb *tcp, int attaching));
@@ -273,9 +293,66 @@ extern int proc_open P((struct tcb *tcp, int attaching));
 extern void tprintf(const char *fmt, ...)
 	__attribute__ ((format (printf, 1, 2)));
 #else
-extern void tprintf(const char *fmt, ...)
+extern void tprintf(const char *fmt, ...);
 #endif
 #else
 extern void tprintf();
 #endif
 
+#ifndef HAVE_STRERROR
+const char *strerror P((int));
+#endif
+#ifndef HAVE_STRSIGNAL
+const char *strsignal P((int));
+#endif
+
+extern int current_personality;
+
+struct sysent {
+	int	nargs;
+	int	sys_flags;
+	int	(*sys_func)();
+	char	*sys_name;
+};
+
+extern struct sysent *sysent;
+extern int nsyscalls;
+
+extern char **errnoent;
+extern int nerrnos;
+
+struct ioctlent {
+	char *doth;
+	char *symbol;
+	unsigned long code;
+};
+
+extern struct ioctlent *ioctlent;
+extern int nioctlent;
+
+extern char **signalent;
+extern int nsignals;
+
+extern struct ioctlent *ioctlent;
+extern int nioctlents;
+extern char **signalent;
+extern int nsignals;
+
+extern struct ioctlent ioctlent0[];
+extern int nioctlents0;
+extern char *signalent0[];
+extern int nsignals0;
+
+#if SUPPORTED_PERSONALITES >= 2
+extern struct ioctlent ioctlent1[];
+extern int nioctlents1;
+extern char *signalent1[];
+extern int nsignals1;
+#endif /* SUPPORTED_PERSONALITIES >= 2 */
+
+#if SUPPORTED_PERSONALITES >= 3
+extern struct ioctlent ioctlent2[];
+extern int nioctlents2;
+extern char *signalent2[];
+extern int nsignals2;
+#endif /* SUPPORTED_PERSONALITIES >= 3 */
