@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 1991, 1992 Paul Kranenburg <pk@cs.few.eur.nl>
  * Copyright (c) 1993 Branko Lankester <branko@hacktic.nl>
- * Copyright (c) 1993, 1994 Rick Sladkey <jrs@world.std.com>
+ * Copyright (c) 1993, 1994, 1995, 1996 Rick Sladkey <jrs@world.std.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -12,11 +12,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by Paul Kranenburg,
- *      Branko Lankester and Rick Sladkey.
- * 4. The name of the author may not be used to endorse or promote products
+ * 3. The name of the author may not be used to endorse or promote products
  *    derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
@@ -30,7 +26,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	desc.c,v 2.21 1994/03/21 05:45:26 jrs Exp
+ *	$Id: desc.c,v 2.30 1996/05/21 03:20:55 jrs Exp $
  */
 
 #include "defs.h"
@@ -71,7 +67,7 @@ static struct xlat fdflags[] = {
 	{ 0,		NULL		},
 };
 
-#ifdef SUNOS4
+#ifdef LOCK_SH
 
 static struct xlat flockcmds[] = {
 	{ LOCK_SH,	"LOCK_SH"	},
@@ -81,7 +77,7 @@ static struct xlat flockcmds[] = {
 	{ 0,		NULL		},
 };
 
-#endif
+#endif /* LOCK_SH */
 
 static struct xlat lockfcmds[] = {
 	{ F_RDLCK,	"F_RDLCK"	},
@@ -158,7 +154,7 @@ struct tcb *tcp;
 	extern struct xlat openmodes[];
 
 	if (entering(tcp)) {
-		tprintf("%d, ", tcp->u_arg[0]);
+		tprintf("%ld, ", tcp->u_arg[0]);
 		printxval(fcntlcmds, tcp->u_arg[1], "F_???");
 		switch (tcp->u_arg[1]) {
 		case F_SETFD:
@@ -167,7 +163,7 @@ struct tcb *tcp;
 				tprintf("0");
 			break;
 		case F_SETOWN: case F_DUPFD:
-			tprintf(", %d", tcp->u_arg[2]);
+			tprintf(", %ld", tcp->u_arg[2]);
 			break;
 		case F_SETFL:
 			tprintf(", ");
@@ -200,33 +196,34 @@ struct tcb *tcp;
 			printflock(tcp, tcp->u_arg[2], 1);
 			break;
 		default:
-			tprintf(", %#x", tcp->u_arg[2]);
+			tprintf(", %#lx", tcp->u_arg[2]);
 			break;
 		}
 	}
 	return 0;
 }
 
-#ifdef SUNOS4
+#ifdef LOCK_SH
+
 int
 sys_flock(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%d, ", tcp->u_arg[0]);
+		tprintf("%ld, ", tcp->u_arg[0]);
 		if (!printflags(flockcmds, tcp->u_arg[1]))
 			tprintf("LOCK_???");
 	}
 	return 0;
 }
-#endif /* SUNOS4 */
+#endif /* LOCK_SH */
 
 int
 sys_close(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%d", tcp->u_arg[0]);
+		tprintf("%ld", tcp->u_arg[0]);
 	}
 	return 0;
 }
@@ -236,7 +233,7 @@ sys_dup(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%d", tcp->u_arg[0]);
+		tprintf("%ld", tcp->u_arg[0]);
 	}
 	return 0;
 }
@@ -246,7 +243,7 @@ sys_dup2(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%d, %d", tcp->u_arg[0], tcp->u_arg[1]);
+		tprintf("%ld, %ld", tcp->u_arg[0], tcp->u_arg[1]);
 	}
 	return 0;
 }
@@ -258,25 +255,17 @@ struct tcb *tcp;
 	return 0;
 }
 
-int
-sys_select(tcp)
+static int
+decode_select(tcp, args)
 struct tcb *tcp;
+long *args;
 {
 	int i, j, nfds;
 	fd_set fds;
 	struct timeval tv;
 	static char outstr[1024];
 	char *sep;
-	int arg;
-#ifdef LINUX
-	int args[5];
-	if (umoven(tcp, tcp->u_arg[0], sizeof args, (char *) args) < 0) {
-		tprintf("[...]");
-		return 0;
-	}
-#else /* !LINUX */
-	int *args = tcp->u_arg;
-#endif /* !LINUX */
+	long arg;
 
 	if (entering(tcp)) {
 		nfds = args[0];
@@ -288,7 +277,7 @@ struct tcb *tcp;
 				continue;
 			}
 			if (!verbose(tcp)) {
-				tprintf(", %#x", arg);
+				tprintf(", %#lx", arg);
 				continue;
 			}
 			if (umove(tcp, arg, &fds) < 0) {
@@ -307,14 +296,16 @@ struct tcb *tcp;
 		if (!args[4])
 			tprintf(", NULL");
 		else if (!verbose(tcp))
-			tprintf(", %#x", args[4]);
+			tprintf(", %#lx", args[4]);
 		else if (umove(tcp, args[4], &tv) < 0)
 			tprintf(", {...}");
-		else
-			tprintf(", {%lu, %lu}", tv.tv_sec, tv.tv_usec);
+		else {
+			tprintf(", {%lu, %lu}",
+				(long) tv.tv_sec, (long) tv.tv_usec);
+		}
 	}
 	else {
-		int cumlen = 0;
+		unsigned int cumlen = 0;
 		char *sep = "";
 
 		if (syserror(tcp))
@@ -346,7 +337,7 @@ struct tcb *tcp;
 					else
 						sprintf(str, " %u", j);
 					cumlen += strlen(str);
-					if (cumlen < sizeof outstr)
+					if (cumlen < sizeof(outstr))
 						strcat(outstr, str);
 					nfds--;
 				}
@@ -363,8 +354,8 @@ struct tcb *tcp;
 
 			if (umove(tcp, args[4], &tv) >= 0)
 				sprintf(str, "%sleft {%lu, %lu}", sep,
-					tv.tv_sec, tv.tv_usec);
-			if ((cumlen += strlen(str)) < sizeof outstr)
+					(long) tv.tv_sec, (long) tv.tv_usec);
+			if ((cumlen += strlen(str)) < sizeof(outstr))
 				strcat(outstr, str);
 		}
 #endif /* LINUX */
@@ -372,3 +363,28 @@ struct tcb *tcp;
 	}
 	return 0;
 }
+
+#ifdef LINUX
+
+int
+sys_oldselect(tcp)
+struct tcb *tcp;
+{
+	long args[5];
+
+	if (umoven(tcp, tcp->u_arg[0], sizeof args, (char *) args) < 0) {
+		tprintf("[...]");
+		return 0;
+	}
+	return decode_select(tcp, args);
+}
+
+#endif /* LINUX */
+
+int
+sys_select(tcp)
+struct tcb *tcp;
+{
+	long *args = tcp->u_arg;
+	return decode_select(tcp, args);
+}
