@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 1991, 1992 Paul Kranenburg <pk@cs.few.eur.nl>
  * Copyright (c) 1993 Branko Lankester <branko@hacktic.nl>
- * Copyright (c) 1993, 1994 Rick Sladkey <jrs@world.std.com>
+ * Copyright (c) 1993, 1994, 1995, 1996 Rick Sladkey <jrs@world.std.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -12,11 +12,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by Paul Kranenburg,
- *      Branko Lankester and Rick Sladkey.
- * 4. The name of the author may not be used to endorse or promote products
+ * 3. The name of the author may not be used to endorse or promote products
  *    derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
@@ -30,7 +26,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	mem.c,v 2.20 1994/03/21 05:45:30 jrs Exp
+ *	$Id: mem.c,v 2.27 1996/05/19 06:40:36 jrs Exp $
  */
 
 #include "defs.h"
@@ -42,7 +38,7 @@ sys_brk(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%#x", tcp->u_arg[0]);
+		tprintf("%#lx", tcp->u_arg[0]);
 	}
 #ifdef linux
 	return RVAL_HEX;
@@ -56,7 +52,7 @@ sys_sbrk(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%u", tcp->u_arg[0]);
+		tprintf("%lu", tcp->u_arg[0]);
 	}
 	return RVAL_HEX;
 }
@@ -69,26 +65,13 @@ static struct xlat mmap_prot[] = {
 	{ PROT_EXEC,	"PROT_EXEC"	},
 	{ 0,		NULL		},
 };
+
 static struct xlat mmap_flags[] = {
 	{ MAP_SHARED,	"MAP_SHARED"	},
 	{ MAP_PRIVATE,	"MAP_PRIVATE"	},
 	{ MAP_FIXED,	"MAP_FIXED"	},
 #ifdef MAP_ANONYMOUS
-/* 	{ MAP_FIXED,	"MAP_ANONYMOUS"	}, */
-       { MAP_ANONYMOUS,        "MAP_ANONYMOUS" },
-#endif
-/* Some Linux specific flags */
-#ifdef MAP_GROWSDOWN
-       { MAP_GROWSDOWN,        "MAP_GROWSDOWN" },
-#endif
-#ifdef MAP_DENYWRITE
-       { MAP_DENYWRITE,        "MAP_DENYWRITE" },
-#endif
-#ifdef MAP_EXECUTABLE
-       { MAP_EXECUTABLE,       "MAP_EXECUTABLE" },
-#endif
-#ifdef MAP_LOCKED
-       { MAP_LOCKED,           "MAP_LOCKED" },
+	{ MAP_ANONYMOUS,"MAP_ANONYMOUS"	},
 #endif
 #ifdef MAP_RENAME
 	{ MAP_RENAME,	"MAP_RENAME"	},
@@ -98,6 +81,15 @@ static struct xlat mmap_flags[] = {
 #endif
 #ifdef _MAP_NEW
 	{ _MAP_NEW,	"_MAP_NEW"	},
+#endif
+#ifdef MAP_GROWSDOWN
+	{ MAP_GROWSDOWN,"MAP_GROWSDOWN"	},
+#endif
+#ifdef MAP_DENYWRITE
+	{ MAP_DENYWRITE,"MAP_DENYWRITE"	},
+#endif
+#ifdef MAP_EXECUTABLE
+	{ MAP_EXECUTABLE,"MAP_EXECUTABLE"},
 #endif
 	{ 0,		NULL		},
 };
@@ -107,22 +99,28 @@ sys_mmap(tcp)
 struct tcb *tcp;
 {
 #ifdef LINUX
-	int u_arg[6];
+#ifdef ALPHA
+	long *u_arg = tcp->u_arg;
+#else /* !ALPHA */
+	long u_arg[6];
+#endif /* !ALPHA */
 #else /* !LINUX */
-	int *u_arg = tcp->u_arg;
+	long *u_arg = tcp->u_arg;
 #endif /* !LINUX */
 
 	if (entering(tcp)) {
 #ifdef LINUX
+#ifndef ALPHA
 		if (umoven(tcp, tcp->u_arg[0], sizeof u_arg,
 				(char *) u_arg) == -1)
 			return 0;
+#endif /* ALPHA */
 #endif /* LINUX */
 
 		/* addr */
-		tprintf("%#x, ", u_arg[0]);
+		tprintf("%#lx, ", u_arg[0]);
 		/* len */
-		tprintf("%u, ", u_arg[1]);
+		tprintf("%lu, ", u_arg[1]);
 		/* prot */
 		printflags(mmap_prot, u_arg[2]);
 		tprintf(", ");
@@ -130,9 +128,9 @@ struct tcb *tcp;
 		printxval(mmap_flags, u_arg[3] & MAP_TYPE, "MAP_???");
 		addflags(mmap_flags, u_arg[3] & ~MAP_TYPE);
 		/* fd */
-		tprintf(", %u, ", u_arg[4]);
+		tprintf(", %ld, ", u_arg[4]);
 		/* offset */
-		tprintf("%#x", u_arg[5]);
+		tprintf("%#lx", u_arg[5]);
 	}
 	return RVAL_HEX;
 }
@@ -142,7 +140,7 @@ sys_munmap(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%#x, %u",
+		tprintf("%#lx, %lu",
 			tcp->u_arg[0], tcp->u_arg[1]);
 	}
 	return 0;
@@ -153,7 +151,7 @@ sys_mprotect(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%#x, %u, ",
+		tprintf("%#lx, %lu, ",
 			tcp->u_arg[0], tcp->u_arg[1]);
 		if (!printflags(mmap_prot, tcp->u_arg[2]))
 			tprintf("PROT_???");
@@ -161,7 +159,36 @@ struct tcb *tcp;
 	return 0;
 }
 
-#ifdef HAVE_MCTL
+#ifdef MS_ASYNC
+
+static struct xlat mctl_sync[] = {
+	{ MS_ASYNC,	"MS_ASYNC"	},
+	{ MS_INVALIDATE,"MS_INVALIDATE"	},
+#ifdef MS_SYNC
+	{ MS_SYNC,	"MS_SYNC"	},
+#endif
+	{ 0,		NULL		},
+};
+
+int
+sys_msync(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		/* addr */
+		tprintf("%#lx", tcp->u_arg[0]);
+		/* len */
+		tprintf(", %lu, ", tcp->u_arg[1]);
+		/* flags */
+		if (!printflags(mctl_sync, tcp->u_arg[2]))
+			tprintf("MS_???");
+	}
+	return 0;
+}
+
+#endif /* MS_ASYNC */
+
+#ifdef MC_SYNC
 
 static struct xlat mctl_funcs[] = {
 	{ MC_LOCK,	"MC_LOCK"	},
@@ -178,12 +205,6 @@ static struct xlat mctl_lockas[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat mctl_sync[] = {
-	{ MS_ASYNC,	"MS_ASYNC"	},
-	{ MS_INVALIDATE,"MS_INVALIDATE"	},
-	{ 0,		NULL		},
-};
-
 int
 sys_mctl(tcp)
 struct tcb *tcp;
@@ -192,9 +213,9 @@ struct tcb *tcp;
 
 	if (entering(tcp)) {
 		/* addr */
-		tprintf("%#x", tcp->u_arg[0]);
+		tprintf("%#lx", tcp->u_arg[0]);
 		/* len */
-		tprintf(", %u, ", tcp->u_arg[1]);
+		tprintf(", %lu, ", tcp->u_arg[1]);
 		/* function */
 		function = tcp->u_arg[2];
 		if (!printflags(mctl_funcs, function))
@@ -219,7 +240,7 @@ struct tcb *tcp;
 	return 0;
 }
 
-#endif /* HAVE_MCTL */
+#endif /* MC_SYNC */
 
 int
 sys_mincore(tcp)
@@ -229,13 +250,13 @@ struct tcb *tcp;
 	char *vec = NULL;
 
 	if (entering(tcp)) {
-		tprintf("%#x, %u, ", tcp->u_arg[0], tcp->u_arg[1]);
+		tprintf("%#lx, %lu, ", tcp->u_arg[0], tcp->u_arg[1]);
 	} else {
 		len = tcp->u_arg[1];
 		if (syserror(tcp) || tcp->u_arg[2] == 0 ||
 			(vec = malloc((u_int)len)) == NULL ||
 			umoven(tcp, tcp->u_arg[2], len, vec) < 0)
-			tprintf("%#x", tcp->u_arg[2]);
+			tprintf("%#lx", tcp->u_arg[2]);
 		else {
 			tprintf("[");
 			for (i = 0; i < len; i++) {
@@ -261,3 +282,4 @@ struct tcb *tcp;
 		return RVAL_HEX;
 	return 0;
 }
+
