@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 1991, 1992 Paul Kranenburg <pk@cs.few.eur.nl>
  * Copyright (c) 1993 Branko Lankester <branko@hacktic.nl>
- * Copyright (c) 1993, 1994 Rick Sladkey <jrs@world.std.com>
+ * Copyright (c) 1993, 1994, 1995, 1996 Rick Sladkey <jrs@world.std.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -12,11 +12,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by Paul Kranenburg,
- *      Branko Lankester and Rick Sladkey.
- * 4. The name of the author may not be used to endorse or promote products
+ * 3. The name of the author may not be used to endorse or promote products
  *    derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
@@ -30,7 +26,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	resource.c,v 2.24 1994/03/21 05:45:35 jrs Exp
+ *	$Id: resource.c,v 2.29 1996/05/19 06:40:37 jrs Exp $
  */
 
 #include "defs.h"
@@ -81,16 +77,16 @@ static struct xlat resources[] = {
 
 static char *
 sprintrlim(lim)
-int lim;
+long lim;
 {
 	static char buf[32];
 
 	if (lim == RLIM_INFINITY)
 		sprintf(buf, "RLIM_INFINITY");
 	else if (lim > 1024 && lim%1024 == 0)
-		sprintf(buf, "%d*1024", lim/1024);
+		sprintf(buf, "%ld*1024", lim/1024);
 	else
-		sprintf(buf, "%d", lim);
+		sprintf(buf, "%ld", lim);
 	return buf;
 }
 
@@ -106,7 +102,7 @@ struct tcb *tcp;
 	}
 	else {
 		if (syserror(tcp) || !verbose(tcp))
-			tprintf("%#x", tcp->u_arg[1]);
+			tprintf("%#lx", tcp->u_arg[1]);
 		else if (umove(tcp, tcp->u_arg[1], &rlim) < 0)
 			tprintf("{...}");
 		else {
@@ -127,7 +123,7 @@ struct tcb *tcp;
 		printxval(resources, tcp->u_arg[0], "RLIMIT_???");
 		tprintf(", ");
 		if (!verbose(tcp))
-			tprintf("%#x", tcp->u_arg[1]);
+			tprintf("%#lx", tcp->u_arg[1]);
 		else if (umove(tcp, tcp->u_arg[1], &rlim) < 0)
 			tprintf("{...}");
 		else {
@@ -149,20 +145,20 @@ static struct xlat usagewho[] = {
 void
 printrusage(tcp, addr)
 struct tcb *tcp;
-int addr;
+long addr;
 {
 	struct rusage ru;
 
 	if (!addr)
 		tprintf("NULL");
 	else if (syserror(tcp) || !verbose(tcp))
-		tprintf("%#x", addr);
+		tprintf("%#lx", addr);
 	else if (umove(tcp, addr, &ru) < 0)
 		tprintf("{...}");
 	else if (!abbrev(tcp)) {
 		tprintf("{ru_utime={%lu, %lu}, ru_stime={%lu, %lu}, ",
-			ru.ru_utime.tv_sec, ru.ru_utime.tv_usec,
-			ru.ru_stime.tv_sec, ru.ru_stime.tv_usec);
+			(long) ru.ru_utime.tv_sec, (long) ru.ru_utime.tv_usec,
+			(long) ru.ru_stime.tv_sec, (long) ru.ru_stime.tv_usec);
 		tprintf("ru_maxrss=%lu, ru_ixrss=%lu, ",
 			ru.ru_maxrss, ru.ru_ixrss);
 		tprintf("ru_idrss=%lu, ru_isrss=%lu, ",
@@ -178,8 +174,8 @@ int addr;
 	}
 	else {
 		tprintf("{ru_utime={%lu, %lu}, ru_stime={%lu, %lu}, ...}",
-			ru.ru_utime.tv_sec, ru.ru_utime.tv_usec,
-			ru.ru_stime.tv_sec, ru.ru_stime.tv_usec);
+			(long) ru.ru_utime.tv_sec, (long) ru.ru_utime.tv_usec,
+			(long) ru.ru_stime.tv_sec, (long) ru.ru_stime.tv_usec);
 	}
 }
 
@@ -208,7 +204,7 @@ struct tcb *tcp;
 
 	if (exiting(tcp)) {
 		if (syserror(tcp) || !verbose(tcp))
-			tprintf("%#x", tcp->u_arg[0]);
+			tprintf("%#lx", tcp->u_arg[0]);
 		else if (umove(tcp, tcp->u_arg[0], &si) < 0)
 			tprintf("{...}");
 		else {
@@ -241,7 +237,7 @@ struct tcb *tcp;
 {
 	if (entering(tcp)) {
 		printxval(priorities, tcp->u_arg[0], "PRIO_???");
-		tprintf(", %u", tcp->u_arg[1]);
+		tprintf(", %lu", tcp->u_arg[1]);
 	}
 	return 0;
 }
@@ -252,7 +248,7 @@ struct tcb *tcp;
 {
 	if (entering(tcp)) {
 		printxval(priorities, tcp->u_arg[0], "PRIO_???");
-		tprintf(", %u, %d", tcp->u_arg[1], tcp->u_arg[2]);
+		tprintf(", %lu, %ld", tcp->u_arg[1], tcp->u_arg[2]);
 	}
 	return 0;
 }
@@ -262,7 +258,7 @@ sys_nice(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp))
-		tprintf("%d", tcp->u_arg[0]);
+		tprintf("%ld", tcp->u_arg[0]);
 	return 0;
 }
 
@@ -278,7 +274,7 @@ struct tcb *tcp;
 		if (tcp->u_arg[0] == 0)
 			tprintf("NULL");
 		else if (syserror(tcp))
-			tprintf("%#x", tcp->u_arg[0]);
+			tprintf("%#lx", tcp->u_arg[0]);
 		else if (umove(tcp, tcp->u_arg[0], &tbuf) < 0)
 			tprintf("{...}");
 		else {
@@ -314,7 +310,7 @@ struct tcb *tcp;
 		printxval(quotacmds, tcp->u_arg[0], "Q_???");
 		tprintf(", ");
 		printstr(tcp, tcp->u_arg[1], -1);
-		tprintf(", %u, %#x", tcp->u_arg[2], tcp->u_arg[3]);
+		tprintf(", %lu, %#lx", tcp->u_arg[2], tcp->u_arg[3]);
 	}
 	return 0;
 }
