@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 1991, 1992 Paul Kranenburg <pk@cs.few.eur.nl>
  * Copyright (c) 1993 Branko Lankester <branko@hacktic.nl>
- * Copyright (c) 1993, 1994 Rick Sladkey <jrs@world.std.com>
+ * Copyright (c) 1993, 1994, 1995, 1996 Rick Sladkey <jrs@world.std.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -12,11 +12,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by Paul Kranenburg,
- *      Branko Lankester and Rick Sladkey.
- * 4. The name of the author may not be used to endorse or promote products
+ * 3. The name of the author may not be used to endorse or promote products
  *    derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
@@ -30,7 +26,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	strace.c,v 2.28 1994/06/29 04:43:25 jrs Exp
+ *	$Id: strace.c,v 2.45 1996/05/19 06:40:37 jrs Exp $
  */
 
 #include "defs.h"
@@ -42,6 +38,9 @@
 #include <sys/resource.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
+#include <pwd.h>
+#include <grp.h>
+
 #ifdef SVR4
 #include <sys/stropts.h>
 #include <poll.h>
@@ -51,14 +50,20 @@ int debug = 0, followfork = 0, followvfork = 0, interactive = 0;
 int rflag = 0, tflag = 0, dtime = 0, cflag = 0;
 int iflag = 0, xflag = 0, qflag = 0;
 int pflag_seen = 0;
+
+char *username = NULL;
+uid_t run_uid;
+gid_t run_gid;
+
 int acolumn = DEFAULT_ACOLUMN;
 int max_strlen = DEFAULT_STRLEN;
-char *outfname;
+char *outfname = NULL;
 FILE *outf;
 struct tcb tcbtab[MAX_PROCS];
 int nprocs;
 char *progname;
 extern char version[];
+extern char **environ;
 
 static struct tcb *pid2tcb P((int pid));
 static int trace P((void));
@@ -108,7 +113,7 @@ int exitval;
 {
 	fprintf(ofp, "\
 usage: strace [-dffhiqrtttTvVxx] [-a column] [-e expr] ... [-o file]\n\
-              [-p pid] ... [-s strsize] [command [arg ...]]\n\
+              [-p pid] ... [-s strsize] [-u username] [command [arg ...]]\n\
    or: strace -c [-e expr] ... [-O overhead] [-S sortby] [command [arg ...]]\n\
 -c -- count time, calls, and errors for each syscall and report summary\n\
 -f -- follow forks, -ff -- with output into separate files\n\
@@ -127,10 +132,20 @@ usage: strace [-dffhiqrtttTvVxx] [-a column] [-e expr] ... [-o file]\n\
 -p pid -- trace process with process id PID, may be repeated\n\
 -s strsize -- limit length of print strings to STRSIZE chars (default %d)\n\
 -S sortby -- sort syscall counts by: time, calls, name, nothing (default %s)\n\
+-u username -- run command as username handling setuid and/or setgid\n\
 ", DEFAULT_ACOLUMN, DEFAULT_STRLEN, DEFAULT_SORTBY);
 	exit(exitval);
 }
 
+#ifdef SVR4
+#ifdef MIPS
+void
+foobar()
+{
+}
+#endif /* MIPS */
+#endif /* SVR4 */
+
 int
 main(argc, argv)
 int argc;
@@ -152,8 +167,9 @@ char *argv[];
 	qualify("verbose=all");
 	qualify("signal=all");
 	set_sortby(DEFAULT_SORTBY);
+	set_personality(DEFAULT_PERSONALITY);
 	while ((c = getopt(argc, argv,
-		"+cdfFhiqrtTvVxa:e:o:O:p:s:S:")) != EOF) {
+		"+cdfFhiqrtTvVxa:e:o:O:p:s:S:u:")) != EOF) {
 		switch (c) {
 		case 'c':
 			cflag++;
@@ -205,10 +221,6 @@ char *argv[];
 			break;
 		case 'o':
 			outfname = strdup(optarg);
-			if ((outf = fopen(outfname, "w")) == NULL) {
-				perror("strace: fopen");
-				exit(1);
-			}
 			break;
 		case 'O':
 			set_overhead(atoi(optarg));
@@ -219,8 +231,12 @@ char *argv[];
 					progname, optarg);
 				break;
 			}
+			if (pid == getpid()) {
+				fprintf(stderr, "%s: I'm sorry, I can't let you do that, Dave.", progname);
+				break;
+			}
 			if ((tcp = alloctcb(pid)) == NULL) {
-				fprintf(stderr, "%s: tcb table full\n",
+				fprintf(stderr, "%s: tcb table full, please recompile strace\n",
 					progname);
 				exit(1);
 			}
@@ -233,13 +249,67 @@ char *argv[];
 		case 'S':
 			set_sortby(optarg);
 			break;
+		case 'u':
+			username = strdup(optarg);
+			break;
 		default:
 			usage(stderr, 1);
 			break;
 		}
 	}
 
-	if (outf == stderr) {
+	/* See if they want to run as another user. */
+	if (username != NULL) {
+		struct passwd *pent;
+
+		if (getuid() != 0 || geteuid() != 0) {
+			fprintf(stderr,
+				"%s: you must be root to use the -u option\n",
+				progname);
+			exit(1);
+		}
+		if ((pent = getpwnam(username)) == NULL) {
+			fprintf(stderr, "%s: cannot find user `%s'\n",
+				progname, optarg);
+			exit(1);
+		}
+		run_uid = pent->pw_uid;
+		run_gid = pent->pw_gid;
+	}
+	else {
+		run_uid = getuid();
+		run_gid = getgid();
+	}
+
+#ifndef SVR4
+	setreuid(geteuid(), getuid());
+#endif
+
+	/* See if they want to pipe the output. */
+	if (outfname && (outfname[0] == '|' || outfname[0] == '!')) {
+		if ((outf = popen(outfname + 1, "w")) == NULL) {
+			fprintf(stderr, "%s: can't popen '%s': %s\n",
+				progname, outfname + 1, strerror(errno));
+			exit(1);
+		}
+		free(outfname);
+		outfname = NULL;
+	}
+
+	/* Check if they want to redirect the output. */
+	if (outfname) {
+		if ((outf = fopen(outfname, "w")) == NULL) {
+			fprintf(stderr, "%s: can't fopen '%s': %s\n",
+				progname, outfname, strerror(errno));
+			exit(1);
+		}
+	}
+
+#ifndef SVR4
+	setreuid(geteuid(), getuid());
+#endif
+
+	if (!outfname) {
 		qflag = 1;
 		setvbuf(outf, buf, _IOLBF, BUFSIZ);
 	}
@@ -249,6 +319,8 @@ char *argv[];
 		qflag = 1;
 
 	for (c = 0, tcp = tcbtab; c < MAX_PROCS; c++, tcp++) {
+		/* Reinitialize the output since it may have changed. */
+		tcp->outf = outf;
 		if (!(tcp->flags & TCB_INUSE) || !(tcp->flags & TCB_ATTACHED))
 			continue;
 #ifdef SVR4
@@ -326,6 +398,13 @@ char *argv[];
 			break;
 		case 0: {
 #ifdef SVR4
+#ifdef MIPS
+			/* Kludge for SGI, see proc_open for details. */
+			sa.sa_handler = foobar;
+			sa.sa_flags = 0;
+			sigemptyset(&sa.sa_mask);
+			sigaction(SIGINT, &sa, NULL);
+#endif /* MIPS */
 			pause();
 #else /* !SVR4 */
 			if (ptrace(PTRACE_TRACEME, 0, (char *) 1, 0) < 0) {
@@ -334,7 +413,38 @@ char *argv[];
 			}
 			if (debug)
 				kill(getpid(), SIGSTOP);
+
+			if (username != NULL || geteuid() == 0) {
+				uid_t run_euid = run_uid;
+				gid_t run_egid = run_gid;
+
+				if (statbuf.st_mode & S_ISUID)
+					run_euid = statbuf.st_uid;
+				if (statbuf.st_mode & S_ISGID)
+					run_egid = statbuf.st_gid;
+
+				/*
+				 * It is important to set groups before we
+				 * lose privileges on setuid.
+				 */
+				if (username != NULL
+				    && initgroups(username, run_gid) < 0) {
+					perror("initgroups");
+					exit(1);
+				}
+				if (setregid(run_gid, run_egid) < 0) {
+					perror("setregid");
+					exit(1);
+				}
+				if (setreuid(run_uid, run_euid) < 0) {
+					perror("setreuid");
+					exit(1);
+				}
+			}
+			else
+				setreuid(run_uid, run_uid);
 #endif /* !SVR4 */
+
 			execv(pathname, &argv[optind]);
 			perror("strace: exec");
 			_exit(1);
@@ -353,6 +463,9 @@ char *argv[];
 				exit(1);
 			}
 #endif /* SVR4 */
+#ifndef SVR4
+			fake_execve(tcp, pathname, &argv[optind], environ);
+#endif
 			break;
 		}
 	}
@@ -370,6 +483,7 @@ char *argv[];
 		sigaddset(&blocked_set, SIGHUP);
 		sigaddset(&blocked_set, SIGINT);
 		sigaddset(&blocked_set, SIGQUIT);
+		sigaddset(&blocked_set, SIGPIPE);
 		sigaddset(&blocked_set, SIGTERM);
 		sa.sa_handler = interrupt;
 #ifdef SUNOS4
@@ -380,6 +494,7 @@ char *argv[];
 	sigaction(SIGHUP, &sa, NULL);
 	sigaction(SIGINT, &sa, NULL);
 	sigaction(SIGQUIT, &sa, NULL);
+	sigaction(SIGPIPE, &sa, NULL);
 	sigaction(SIGTERM, &sa, NULL);
 #ifdef SVR4
 	sa.sa_handler = reaper;
@@ -401,7 +516,14 @@ struct tcb *tcp;
 
 	if (outfname && followfork > 1) {
 		sprintf(name, "%s.%u", outfname, tcp->pid);
-		if ((fp = fopen(name, "w")) == NULL) {
+#ifndef SVR4
+		setreuid(geteuid(), getuid());
+#endif
+		fp = fopen(name, "w");
+#ifndef SVR4
+		setreuid(geteuid(), getuid());
+#endif
+		if (fp == NULL) {
 			perror("fopen");
 			return;
 		}
@@ -446,10 +568,12 @@ int attaching;
 	sysset_t sc_enter, sc_exit;
 	sigset_t signals;
 	fltset_t faults;
+#ifndef MIPS
 	prrun_t run;
+#endif
 #ifndef HAVE_POLLABLE_PROCFS
 	static int last_pfd;
-#endif /* !HAVE_POLLABLE_PROCFS */
+#endif
 
 	/* Open the process pseudo-file in /proc. */
 	sprintf(proc, "/proc/%d", tcp->pid);
@@ -508,7 +632,7 @@ int attaching;
 		return -1;
 	}
 #endif /* !PIOCSET */
-	/* Enable all syscall enties. */
+	/* Enable all syscall entries. */
 	prfillset(&sc_enter);
 	if (ioctl(tcp->pfd, PIOCSENTRY, &sc_enter) < 0) {
 		perror("PIOCSENTRY");
@@ -533,21 +657,40 @@ int attaching;
 		return -1;
 	}
 	if (!attaching) {
+#ifdef MIPS
+		/*
+		 * The SGI PRSABORT doesn't work for pause() so
+		 * we send it a caught signal to wake it up.
+		 */
+		kill(tcp->pid, SIGINT);
+#else /* !MIPS */
 		/* The child is in a pause(), abort it. */
 		run.pr_flags = PRSABORT;
 		if (ioctl(tcp->pfd, PIOCRUN, &run) < 0) {
 			perror("PIOCRUN");
 			return -1;
 		}
-		/* Wait for the child to come out of the system call. */
-		if (ioctl(tcp->pfd, PIOCWSTOP, &tcp->status) < 0) {
-			perror("PIOCWSTOP");
-			return -1;
-		}
-		/* Then set it running: execve should be next. */
-		if (ioctl(tcp->pfd, PIOCRUN, NULL) < 0) {
-			perror("PIOCRUN");
-			return -1;
+#endif /* !MIPS */
+		for (;;) {
+			/* Wait for the child to do something. */
+			if (ioctl(tcp->pfd, PIOCWSTOP, &tcp->status) < 0) {
+				perror("PIOCWSTOP");
+				return -1;
+			}
+			if (tcp->status.pr_why == PR_SYSENTRY) {
+#ifdef HAVE_PR_SYSCALL
+				int scno = tcp->status.pr_syscall;
+#else /* !HAVE_PR_SYSCALL */
+				int scno = tcp->status.pr_what;
+#endif /* !HAVE_PR_SYSCALL */
+				if (scno == SYS_execve)
+					break;
+			}
+			/* Set it running: maybe execve will be next. */
+			if (ioctl(tcp->pfd, PIOCRUN, NULL) < 0) {
+				perror("PIOCRUN");
+				return -1;
+			}
 		}
 	}
 #ifndef HAVE_POLLABLE_PROCFS
@@ -615,7 +758,7 @@ struct tcb *tcp;
 		tcp->pfd = -1;
 #ifdef SVR4
 		rebuild_pollv();
-#endif /* SVR4 */
+#endif
 	}
 	if (tcp->parent != NULL) {
 		tcp->parent->nchildren--;
@@ -625,7 +768,7 @@ struct tcb *tcp;
 	if (tcp->outf != stderr)
 		fclose(tcp->outf);
 #endif
-	tcp->outf = stderr;
+	tcp->outf = 0;
 }
 
 #ifndef SVR4
@@ -655,14 +798,17 @@ struct tcb *tcp;
 
 #endif /* !SVR4 */
 
+/* detach traced process; continue with sig */
+
 static int
 detach(tcp, sig)
 struct tcb *tcp;
 int sig;
 {
-	/* detach traced process; continue with sig */
-	int error;
+	int error = 0;
+#ifdef LINUX
 	int status;
+#endif
 
 	if (tcp->flags & TCB_BPTSET)
 		sig = SIGKILL;
@@ -696,7 +842,7 @@ int sig;
 				break;
 			}
 			if (!WIFSTOPPED(status)) {
-				/* Au revoir, mon ami. */	
+				/* Au revoir, mon ami. */
 				break;
 			}
 			if (WSTOPSIG(status) == SIGSTOP) {
@@ -751,7 +897,7 @@ int sig;
 
 	while ((pid = waitpid(-1, &status, WNOHANG)) > 0) {
 #if 0
-	struct tcb *tcp;
+		struct tcb *tcp;
 
 		tcp = pid2tcb(pid);
 		if (tcp)
@@ -781,8 +927,10 @@ cleanup()
 		}
 		if (tcp->flags & TCB_ATTACHED)
 			detach(tcp, 0);
-		else
+		else {
+			kill(tcp->pid, SIGCONT);
 			kill(tcp->pid, SIGTERM);
+		}
 	}
 	if (cflag)
 		call_summary(outf);
@@ -795,18 +943,43 @@ int sig;
 	interrupted = 1;
 }
 
-#ifndef HAVE_STRSIGNAL
+#ifndef HAVE_STRERROR
 
-char *
-strsignal(sig)
-int sig;
+#ifndef SYS_ERRLIST_DECLARED
+extern int sys_nerr;
+extern char *sys_errlist[];
+#endif /* SYS_ERRLIST_DECLARED */
+
+const char *
+strerror(errno)
+int errno;
 {
 	static char buf[64];
+
+	if (errno < 1 || errno >= sys_nerr) {
+		sprintf(buf, "Unknown error %d", errno);
+		return buf;
+	}
+	return sys_errlist[errno];
+}
+
+#endif /* HAVE_STERRROR */
+
+#ifndef HAVE_STRSIGNAL
+
+#ifndef SYS_SIGLIST_DECLARED
 #ifdef HAVE__SYS_SIGLIST
 	extern char *_sys_siglist[];
 #else
 	extern char *sys_siglist[];
 #endif
+#endif /* SYS_SIGLIST_DECLARED */
+
+const char *
+strsignal(sig)
+int sig;
+{
+	static char buf[64];
 
 	if (sig < 1 || sig >= NSIG) {
 		sprintf(buf, "Unknown signal %d", sig);
@@ -925,6 +1098,7 @@ int pfd;
 	sigaction(SIGHUP, &sa, NULL);
 	sigaction(SIGINT, &sa, NULL);
 	sigaction(SIGQUIT, &sa, NULL);
+	sigaction(SIGPIPE, &sa, NULL);
 	sigaction(SIGTERM, &sa, NULL);
 	sa.sa_handler = wakeup_handler;
 	sigaction(SIGUSR1, &sa, NULL);
@@ -942,7 +1116,7 @@ int pfd;
 		if (i != pfd && i != proc_poll_pipe[1])
 			close(i);
 	}
-			
+
 	pollinfo.fd = pfd;
 	pollinfo.pid = getpid();
 	for (;;) {
@@ -1017,19 +1191,20 @@ trace()
 	struct tcb *tcp;
 	int pfd;
 	int what;
-	int ioctl_result, ioctl_errno;
+	int ioctl_result = 0, ioctl_errno = 0;
 
 	for (;;) {
 		if (interactive)
 			sigprocmask(SIG_SETMASK, &empty_set, NULL);
 
+		if (nprocs == 0)
+			break;
+
 		switch (nprocs) {
-		case 0:
-			return 0;
 		case 1:
 #ifndef HAVE_POLLABLE_PROCFS
 			if (proc_poll_pipe[0] == -1) {
-#endif /* !HAVE_POLLABLE_PROCFS */
+#endif
 				tcp = pid2tcb(0);
 				if (!tcp)
 					continue;
@@ -1121,7 +1296,7 @@ trace()
 		case PR_REQUESTED:
 			if (tcp->status.pr_flags & PR_ASLEEP) {
 				tcp->status.pr_why = PR_SYSENTRY;
-				if (syscall(tcp) < 0) {
+				if (trace_syscall(tcp) < 0) {
 					fprintf(stderr, "syscall trouble\n");
 					exit(1);
 				}
@@ -1129,7 +1304,7 @@ trace()
 			break;
 		case PR_SYSENTRY:
 		case PR_SYSEXIT:
-			if (syscall(tcp) < 0) {
+			if (trace_syscall(tcp) < 0) {
 				fprintf(stderr, "syscall trouble\n");
 				exit(1);
 			}
@@ -1175,7 +1350,7 @@ trace()
 	struct rusage ru;
 #endif /* LINUX */
 
-	for (;;) {
+	while (nprocs != 0) {
 		if (interactive)
 			sigprocmask(SIG_SETMASK, &empty_set, NULL);
 #ifdef LINUX
@@ -1277,7 +1452,7 @@ trace()
 
 		if (tcp->flags & TCB_STARTUP) {
 			/*
-			 * This flag is there to keep us in sync
+			 * This flag is there to keep us in sync.
 			 * Next time this process stops it should
 			 * really be entering a system call.
 			 */
@@ -1351,7 +1526,7 @@ trace()
 			tcp->flags &= ~TCB_SUSPENDED;
 			continue;
 		}
-		if (syscall(tcp) < 0) {
+		if (trace_syscall(tcp) < 0) {
 			if (tcp->flags & TCB_ATTACHED)
 				detach(tcp, 0);
 			else {
@@ -1410,7 +1585,8 @@ va_dcl
 	va_list args;
 
 	VA_START(args, fmt);
-	curcol += vfprintf(outf, fmt, args);
+	if (outf)
+		curcol += vfprintf(outf, fmt, args);
 	va_end(args);
 	return;
 }
@@ -1424,7 +1600,7 @@ struct tcb *tcp;
 		tprintf(" <unfinished ...>\n");
 	}
 	curcol = 0;
-	if (followfork == 1 && outfname)
+	if ((followfork == 1 || pflag_seen > 1) && outfname)
 		tprintf("%-5d ", tcp->pid);
 	else if (nprocs > 1 && !outfname)
 		tprintf("[pid %5u] ", tcp->pid);
@@ -1438,15 +1614,19 @@ struct tcb *tcp;
 			if (otv.tv_sec == 0)
 				otv = tv;
 			tv_sub(&dtv, &tv, &otv);
-			tprintf("%6ld.%06ld ", dtv.tv_sec, dtv.tv_usec);
+			tprintf("%6ld.%06ld ",
+				(long) dtv.tv_sec, (long) dtv.tv_usec);
 			otv = tv;
 		}
-		else if (tflag > 2)
-			tprintf("%ld.%06ld ", tv.tv_sec, tv.tv_usec);
+		else if (tflag > 2) {
+			tprintf("%ld.%06ld ",
+				(long) tv.tv_sec, (long) tv.tv_usec);
+		}
 		else {
-			strftime(str, sizeof(str), "%T", localtime(&tv.tv_sec));
+			time_t local = tv.tv_sec;
+			strftime(str, sizeof(str), "%T", localtime(&local));
 			if (tflag > 1)
-				tprintf("%s.%06ld ", str, tv.tv_usec);
+				tprintf("%s.%06ld ", str, (long) tv.tv_usec);
 			else
 				tprintf("%s ", str);
 		}
