@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1993, 1994 Rick Sladkey <jrs@world.std.com>
+ * Copyright (c) 1993, 1994, 1995, 1996 Rick Sladkey <jrs@world.std.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -10,11 +10,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by Paul Kranenburg,
- *      Branko Lankester and Rick Sladkey.
- * 4. The name of the author may not be used to endorse or promote products
+ * 3. The name of the author may not be used to endorse or promote products
  *    derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
@@ -28,12 +24,12 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	stream.c,v 2.6 1994/03/21 05:45:42 jrs Exp
+ *	$Id: stream.c,v 2.11 1996/05/21 03:20:57 jrs Exp $
  */
 
 #include "defs.h"
 
-#ifdef HAVE_POLL
+#ifdef HAVE_SYS_STREAM_H
 
 #include <stropts.h>
 #include <poll.h>
@@ -103,7 +99,7 @@ struct tcb *tcp;
 
 	if (entering(tcp)) {
 		/* fd */
-		tprintf("%d, ", tcp->u_arg[0]);
+		tprintf("%ld, ", tcp->u_arg[0]);
 		/* control and data */
 		for (i = 1; i < 3; i++)
 			printstrbufarg(tcp, tcp->u_arg[i], 0);
@@ -122,10 +118,10 @@ struct tcb *tcp;
 
 	if (entering(tcp)) {
 		/* fd */
-		tprintf("%u, ", tcp->u_arg[0]);
+		tprintf("%lu, ", tcp->u_arg[0]);
 	} else {
 		if (syserror(tcp)) {
-			tprintf("%#x, %#x, %#x",
+			tprintf("%#lx, %#lx, %#lx",
 				tcp->u_arg[1], tcp->u_arg[2], tcp->u_arg[3]);
 			return 0;
 		}
@@ -179,12 +175,12 @@ struct tcb *tcp;
 
 	if (entering(tcp)) {
 		/* fd */
-		tprintf("%d, ", tcp->u_arg[0]);
+		tprintf("%ld, ", tcp->u_arg[0]);
 		/* control and data */
 		for (i = 1; i < 3; i++)
 			printstrbufarg(tcp, tcp->u_arg[i], 0);
 		/* band */
-		tprintf("%d, ", tcp->u_arg[3]);
+		tprintf("%ld, ", tcp->u_arg[3]);
 		/* flags */
 		if (!printflags(pmsgflags, tcp->u_arg[4]))
 			tprintf("0");
@@ -200,10 +196,10 @@ struct tcb *tcp;
 
 	if (entering(tcp)) {
 		/* fd */
-		tprintf("%u, ", tcp->u_arg[0]);
+		tprintf("%lu, ", tcp->u_arg[0]);
 	} else {
 		if (syserror(tcp)) {
-			tprintf("%#x, %#x, %#x, %#x", tcp->u_arg[1],
+			tprintf("%#lx, %#lx, %#lx, %#lx", tcp->u_arg[1],
 				tcp->u_arg[2], tcp->u_arg[3], tcp->u_arg[4]);
 			return 0;
 		}
@@ -277,20 +273,20 @@ struct tcb *tcp;
 		int nfds = tcp->u_arg[1];
 
 		if (nfds <= 0) {
-			tprintf("%#x, %d, %d\n",
+			tprintf("%#lx, %d, %ld\n",
 				tcp->u_arg[0], nfds, tcp->u_arg[2]);
 			return 0;
 		}
 		pollp = (struct pollfd *) malloc(nfds * sizeof(*pollp));
 		if (pollp == NULL) {
 			fprintf(stderr, "sys_poll: no memory\n");
-			tprintf("%#x, %d, %d",
+			tprintf("%#lx, %d, %ld",
 				tcp->u_arg[0], nfds, tcp->u_arg[2]);
 			return 0;
 		}
 		if (umoven(tcp, tcp->u_arg[0],
 			   (nfds * sizeof(*pollp)), (char *) pollp) < 0) {
-			tprintf("%#x", tcp->u_arg[0]);
+			tprintf("%#lx", tcp->u_arg[0]);
 		}
 		else {
 			tprintf("[");
@@ -318,7 +314,7 @@ struct tcb *tcp;
 		if (tcp->u_arg[2] == INFTIM)
 			tprintf("INFTIM");
 		else
-			tprintf("%d", tcp->u_arg[2]);
+			tprintf("%ld", tcp->u_arg[2]);
 		free(pollp);
 	}
 	return 0;
@@ -403,6 +399,7 @@ static struct xlat stream_atmark_options[] = {
 };
 #endif /* I_ATMARK */
 
+#ifdef TI_BIND
 static struct xlat transport_user_options[] = {
 	{ T_CONN_REQ,	"T_CONN_REQ"	},
 	{ T_CONN_RES,	"T_CONN_RES"	},
@@ -434,6 +431,7 @@ static struct xlat transport_provider_options[] = {
 	{ T_ORDREL_IND,	"T_ORDREL_IND"	},
 	{ 0,		NULL		},
 };
+#endif /* TI_BIND */
 
 static int
 internal_stream_ioctl(tcp, arg)
@@ -795,4 +793,4 @@ int code, arg;
 	}
 }
 
-#endif /* !HAVE_POLL */
+#endif /* HAVE_SYS_STREAM_H */
