@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 1991, 1992 Paul Kranenburg <pk@cs.few.eur.nl>
  * Copyright (c) 1993 Branko Lankester <branko@hacktic.nl>
- * Copyright (c) 1993, 1994 Rick Sladkey <jrs@world.std.com>
+ * Copyright (c) 1993, 1994, 1995, 1996 Rick Sladkey <jrs@world.std.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -12,11 +12,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by Paul Kranenburg,
- *      Branko Lankester and Rick Sladkey.
- * 4. The name of the author may not be used to endorse or promote products
+ * 3. The name of the author may not be used to endorse or promote products
  *    derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
@@ -30,17 +26,8 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	syscall.c,v 2.25 1994/03/12 23:11:57 jrs Exp
+ *	$Id: syscall.c,v 2.42 1996/05/20 05:39:41 jrs Exp $
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#ifdef LINUX
-#define __KERNEL__
-#include <errno.h>
-#undef __KERNEL__
-#endif
 
 #include "defs.h"
 
@@ -51,22 +38,136 @@
 #include <sys/syscall.h>
 #include <sys/param.h>
 
-#ifndef LINUX
-extern char *sys_errlist[];
+#ifndef SYS_ERRLIST_DECLARED
 extern int sys_nerr;
-#endif /* !LINUX */
+extern char *sys_errlist[];
+#endif /* SYS_ERRLIST_DECLARED */
+
+#ifdef LINUX
+#ifndef ERESTARTSYS
+#define ERESTARTSYS	512
+#endif
+#ifndef ERESTARTNOINTR
+#define ERESTARTNOINTR	513
+#endif
+#ifndef ERESTARTNOHAND
+#define ERESTARTNOHAND	514	/* restart if no handler.. */
+#endif
+#ifndef ENOIOCTLCMD
+#define ENOIOCTLCMD	515	/* No ioctl command */
+#endif
+#endif /* LINUX */
 
 #include "syscall.h"
 
-struct sysent sysent[] = {
+/* Define these shorthand notations to simplify the syscallent files. */
+#define TF TRACE_FILE
+#define TI TRACE_IPC
+#define TN TRACE_NETWORK
+#define TP TRACE_PROCESS
+#define TS TRACE_SIGNAL
+
+struct sysent sysent0[] = {
 #include "syscallent.h"
 };
-int nsyscalls = sizeof sysent / sizeof sysent[0];
+int nsyscalls0 = sizeof sysent0 / sizeof sysent0[0];
 
-char *errnoent[] = {
+#if SUPPORTED_PERSONALITIES >= 2
+struct sysent sysent1[] = {
+#include "syscallent1.h"
+};
+int nsyscalls1 = sizeof sysent1 / sizeof sysent1[0];
+#endif /* SUPPORTED_PERSONALITIES >= 2 */
+
+#if SUPPORTED_PERSONALITIES >= 3
+struct sysent sysent2[] = {
+#include "syscallent2.h"
+};
+int nsyscalls1 = sizeof sysent2 / sizeof sysent2[0];
+#endif /* SUPPORTED_PERSONALITIES >= 3 */
+
+struct sysent *sysent;
+int nsyscalls;
+
+/* Now undef them since short defines cause wicked namespace pollution. */
+#undef TF
+#undef TI
+#undef TN
+#undef TP
+#undef TS
+
+char *errnoent0[] = {
 #include "errnoent.h"
 };
-int nerrnos = sizeof errnoent / sizeof errnoent[0];
+int nerrnos0 = sizeof errnoent0 / sizeof errnoent0[0];
+
+#if SUPPORTED_PERSONALITIES >= 2
+char *errnoent1[] = {
+#include "errnoent1.h"
+};
+int nerrnos1 = sizeof errnoent1 / sizeof errnoent1[0];
+#endif /* SUPPORTED_PERSONALITIES >= 2 */
+
+#if SUPPORTED_PERSONALITIES >= 3
+char *errnoent2[] = {
+#include "errnoent2.h"
+};
+int nerrnos2 = sizeof errnoent2 / sizeof errnoent2[0];
+#endif /* SUPPORTED_PERSONALITIES >= 3 */
+
+char **errnoent;
+int nerrnos;
+
+int current_personality;
+
+int
+set_personality(int personality)
+{
+	switch (personality) {
+	case 0:
+		errnoent = errnoent0;
+		nerrnos = nerrnos0;
+		sysent = sysent0;
+		nsyscalls = nsyscalls0;
+		ioctlent = ioctlent0;
+		nioctlents = nioctlents0;
+		signalent = signalent0;
+		nsignals = nsignals0;
+		break;
+
+#if SUPPORTED_PERSONALITIES >= 2
+	case 1:
+		errnoent = errnoent1;
+		nerrnos = nerrnos1;
+		sysent = sysent1;
+		nsyscalls = nsyscalls1;
+		ioctlent = ioctlent1;
+		nioctlents = nioctlents1;
+		signalent = signalent1;
+		nsignals = nsignals1;
+		break;
+#endif /* SUPPORTED_PERSONALITIES >= 2 */
+
+#if SUPPORTED_PERSONALITIES >= 3
+	case 1:
+		errnoent = errnoent2;
+		nerrnos = nerrnos2;
+		sysent = sysent2;
+		nsyscalls = nsyscalls2;
+		ioctlent = ioctlent2;
+		nioctlents = nioctlents2;
+		signalent = signalent2;
+		nsignals = nsignals2;
+		break;
+#endif /* SUPPORTED_PERSONALITIES >= 3 */
+
+	default:
+		return -1;
+	}
+
+	current_personality = personality;
+	return 0;
+}
 
 int qual_flags[MAX_QUALS];
 
@@ -159,6 +260,23 @@ char *s;
 	return -1;
 }
 
+static int
+lookup_class(s)
+char *s;
+{
+	if (strcmp(s, "file") == 0)
+		return TRACE_FILE;
+	if (strcmp(s, "ipc") == 0)
+		return TRACE_IPC;
+	if (strcmp(s, "network") == 0)
+		return TRACE_NETWORK;
+	if (strcmp(s, "process") == 0)
+		return TRACE_PROCESS;
+	if (strcmp(s, "signal") == 0)
+		return TRACE_SIGNAL;
+	return -1;
+}
+
 void
 qualify(s)
 char *s;
@@ -202,6 +320,17 @@ char *s;
 			qual_flags[i] &= ~opt->bitflag;
 	}
 	for (p = strtok(s, ","); p; p = strtok(NULL, ",")) {
+		if (opt->bitflag == QUAL_TRACE && (n = lookup_class(p)) > 0) {
+			for (i = 0; i < MAX_QUALS; i++) {
+				if (sysent[i].sys_flags & n) {
+					if (not)
+						qual_flags[i] &= ~opt->bitflag;
+					else
+						qual_flags[i] |= opt->bitflag;
+				}
+			}
+			continue;
+		}
 		if ((n = (*opt->lookup)(p)) < 0) {
 			fprintf(stderr, "strace: invalid %s `%s'\n",
 				opt->argument_name, p);
@@ -221,6 +350,8 @@ struct tcb *tcp;
 {
 	if (syserror(tcp))
 		return;
+	if (tcp->u_arg[0] < 0 || tcp->u_arg[0] >= MAX_QUALS)
+		return;
 	switch (tcp->scno) {
 	case SYS_read:
 #ifdef SYS_recv
@@ -245,10 +376,10 @@ struct tcb *tcp;
 	}
 }
 
-#ifndef SUNOS4
-
 enum subcall_style { shift_style, deref_style, mask_style };
 
+#if !(defined(LINUX) && defined(ALPHA))
+
 static void
 decode_subcall(tcp, subcall, nsubcalls, style)
 struct tcb *tcp;
@@ -292,21 +423,83 @@ enum subcall_style style;
 		break;
 	}
 }
-
-#endif /* !SUNOS4 */
+#endif
 
 struct tcb *tcp_last = NULL;
 
+static int
+internal_syscall(tcp)
+struct tcb *tcp;
+{
+	/*
+	 * We must always trace a few critical system calls in order to
+	 * correctly support following forks in the presence of tracing
+	 * qualifiers.
+	 */
+	switch (tcp->scno) {
+
+#ifdef SYS_fork
+	case SYS_fork:
+#endif
+#ifdef SYS_vfork
+	case SYS_vfork:
+#endif
+		internal_fork(tcp);
+		break;
+
+#ifdef SYS_execv
+	case SYS_execv:
+#endif
+#ifdef SYS_execve
+	case SYS_execve:
+#endif
+		internal_exec(tcp);
+		break;
+
+#ifdef SYS_wait
+	case SYS_wait:
+#endif
+#ifdef SYS_wait4
+	case SYS_wait4:
+#endif
+#ifdef SYS_waitpid
+	case SYS_waitpid:
+#endif
+#ifdef SYS_waitsys
+	case SYS_waitsys:
+#endif
+		internal_wait(tcp);
+		break;
+
+#ifdef SYS_exit
+	case SYS_exit:
+#endif
+		internal_exit(tcp);
+		break;
+	}
+	return 0;
+}
+
 int
-syscall(tcp)
+trace_syscall(tcp)
 struct tcb *tcp;
 {
-	int i;
 	int sys_res;
 	struct timeval tv;
-	int scno;
+	long scno;
 #ifdef LINUX
-	int eax;
+#ifdef I386
+	long eax;
+#else /* !I386 */
+#ifdef M68K
+	int d0;
+#else /* !M68K */
+#ifdef ALPHA
+	long r0;
+	long a3;
+#endif /* ALPHA */
+#endif /* M68K */
+#endif /* !I386 */
 #endif /* LINUX */
 #ifndef SVR4
 	int pid = tcp->pid;
@@ -316,9 +509,48 @@ struct tcb *tcp;
 	if (dtime && (tcp->flags & TCB_INSYSCALL))
 		gettimeofday(&tv, NULL);
 #ifdef LINUX
+#ifdef I386
 	if (upeek(pid, 4*ORIG_EAX, &scno) < 0)
 		return -1;
-#endif
+#else /* !I386 */
+#ifdef M68K
+	if (upeek(pid, 4*PT_ORIG_D0, &scno) < 0)
+		return -1;
+#else /* !M68K */
+#ifdef ALPHA
+	if (upeek(pid, REG_A3, &a3) < 0)
+		return -1;
+
+	if (!(tcp->flags & TCB_INSYSCALL)) {
+		if (upeek(pid, REG_R0, &scno) < 0)
+			return -1;
+
+		/* Check if we return from execve. */
+		if (scno == 0 && tcp->flags & TCB_WAITEXECVE) {
+			tcp->flags &= ~TCB_WAITEXECVE;
+			return 0;
+		}
+
+		/*
+		 * Do some sanity checks to figure out if it's
+		 * really a syscall entry
+		 */
+		if (scno < 0 || scno > nsyscalls) {
+			if (a3 == 0 || a3 == -1) {
+				if (debug)
+					fprintf (stderr, "stray syscall exit: r0 = %ld\n", scno);
+				return 0;
+			}
+		}
+	}
+	else {
+		if (upeek(pid, REG_R0, &r0) < 0)
+			return -1;
+	}
+#endif /* ALPHA */
+#endif /* !M68K */
+#endif /* !I386 */
+#endif /* LINUX */
 #ifdef SUNOS4
 	if (upeek(pid, uoff(u_arg[7]), &scno) < 0)
 		return -1;
@@ -331,10 +563,15 @@ struct tcb *tcp;
 #endif /* !HAVE_PR_SYSCALL */
 	if (!(tcp->flags & TCB_INSYSCALL)) {
 		if (tcp->status.pr_why != PR_SYSENTRY) {
-			if (scno == SYS_fork || scno == SYS_vfork) {
+			if (
+			    scno == SYS_fork
+#ifdef SYS_vfork
+			    || scno == SYS_vfork
+#endif /* SYS_vfork */
+			    ) {
 				/* We are returning in the child, fake it. */
 				tcp->status.pr_why = PR_SYSENTRY;
-				syscall(tcp);
+				trace_syscall(tcp);
 				tcp->status.pr_why = PR_SYSEXIT;
 			}
 			else {
@@ -372,32 +609,33 @@ struct tcb *tcp;
 	}
 #endif /* SUNOS4 */
 #ifdef LINUX
+#ifdef I386
 	if (upeek(pid, 4*EAX, &eax) < 0)
 		return -1;
 	if (eax != -ENOSYS && !(tcp->flags & TCB_INSYSCALL)) {
 		if (debug)
-			fprintf(stderr, "stray syscall exit: eax = %d\n", eax);
+			fprintf(stderr, "stray syscall exit: eax = %ld\n", eax);
 		return 0;
 	}
+#else /* !I386 */
+#ifdef M68K
+	if (upeek(pid, 4*PT_D0, &d0) < 0)
+		return -1;
+	if (d0 != -ENOSYS && !(tcp->flags & TCB_INSYSCALL)) {
+		if (debug)
+			fprintf(stderr, "stray syscall exit: d0 = %ld\n", d0);
+		return 0;
+	}
+#endif /* M68K */
+#endif /* !I386 */
 #endif /* LINUX */
-	if (tcp->flags & TCB_INSYSCALL) {
-		unsigned int u_error;
 
-		if (!(qual_flags[tcp->scno] & QUAL_TRACE)) {
-			tcp->flags &= ~TCB_INSYSCALL;
-			return 0;
-		}
-		if (tcp->flags & TCB_REPRINT) {
-			printleader(tcp);
-			tprintf("<... ");
-			if (tcp->scno >= nsyscalls)
-				tprintf("syscall_%u", tcp->scno);
-			else
-				tprintf("%s", sysent[tcp->scno].sys_name);
-			tprintf(" resumed> ");
-		}
+	if (tcp->flags & TCB_INSYSCALL) {
+		long u_error;
+
 #ifdef LINUX
-		if (eax && (unsigned) -eax < nerrnos) {
+#ifdef I386
+		if (eax < 0 && -eax < nerrnos) {
 			tcp->u_rval = -1;
 			u_error = -eax;
 		}
@@ -405,6 +643,29 @@ struct tcb *tcp;
 			tcp->u_rval = eax;
 			u_error = 0;
 		}
+#else /* !I386 */
+#ifdef M68K
+		if (d0 && (unsigned) -d0 < nerrnos) {
+			tcp->u_rval = -1;
+			u_error = -d0;
+		}
+		else {
+			tcp->u_rval = d0;
+			u_error = 0;
+		}
+#else /* !M68K */
+#ifdef ALPHA
+		if (a3) {
+			tcp->u_rval = -1;
+			u_error = r0;
+		}
+		else {
+			tcp->u_rval = r0;
+			u_error = 0;
+		}
+#endif /* ALPHA */
+#endif /* !M68K */
+#endif /* !I386 */
 #endif /* LINUX */
 #ifdef SUNOS4
 		/* get error code from user struct */
@@ -439,8 +700,35 @@ struct tcb *tcp;
 			u_error = 0;
 		}
 #endif /* I386 */
+#ifdef MIPS
+		if (tcp->status.pr_reg[CTX_A3]) {
+			tcp->u_rval = -1;
+			u_error = tcp->status.pr_reg[CTX_V0];
+		}
+		else {
+			tcp->u_rval = tcp->status.pr_reg[CTX_V0];
+			u_error = 0;
+		}
+#endif /* MIPS */
 #endif /* SVR4 */
 		tcp->u_error = u_error;
+
+		internal_syscall(tcp);
+		if (!(qual_flags[tcp->scno] & QUAL_TRACE)) {
+			tcp->flags &= ~TCB_INSYSCALL;
+			return 0;
+		}
+
+		if (tcp->flags & TCB_REPRINT) {
+			printleader(tcp);
+			tprintf("<... ");
+			if (tcp->scno >= nsyscalls)
+				tprintf("syscall_%lu", tcp->scno);
+			else
+				tprintf("%s", sysent[tcp->scno].sys_name);
+			tprintf(" resumed> ");
+		}
+
 		if (cflag) {
 			call_count[tcp->scno]++;
 			if (u_error)
@@ -479,9 +767,9 @@ struct tcb *tcp;
 		tabto(acolumn);
 		if (qual_flags[tcp->scno] & QUAL_RAW) {
 			if (u_error)
-				tprintf("= -1 (errno %d)", u_error);
+				tprintf("= -1 (errno %ld)", u_error);
 			else
-				tprintf("= %#x", tcp->u_rval);
+				tprintf("= %#lx", tcp->u_rval);
 		}
 		else if (!(sys_res & RVAL_NONE) && u_error) {
 #ifdef LINUX
@@ -502,13 +790,13 @@ struct tcb *tcp;
 					tprintf("%s (%s)", errnoent[u_error],
 						sys_errlist[u_error]);
 				else if (u_error < nerrnos)
-					tprintf("%s (errno %d)",
+					tprintf("%s (errno %ld)",
 						errnoent[u_error], u_error);
 				else if (u_error < sys_nerr)
-					tprintf("ERRNO_%d (%s)", u_error,
+					tprintf("ERRNO_%ld (%s)", u_error,
 						sys_errlist[u_error]);
 				else
-					tprintf("E??? (errno %d)", u_error);
+					tprintf("E??? (errno %ld)", u_error);
 #ifdef LINUX
 				break;
 			}
@@ -520,16 +808,16 @@ struct tcb *tcp;
 			else {
 				switch (sys_res & RVAL_MASK) {
 				case RVAL_HEX:
-					tprintf("= %#x", tcp->u_rval);
+					tprintf("= %#lx", tcp->u_rval);
 					break;
 				case RVAL_OCTAL:
-					tprintf("= %#o", tcp->u_rval);
+					tprintf("= %#lo", tcp->u_rval);
 					break;
 				case RVAL_UDECIMAL:
-					tprintf("= %u", tcp->u_rval);
+					tprintf("= %lu", tcp->u_rval);
 					break;
 				case RVAL_DECIMAL:
-					tprintf("= %d", tcp->u_rval);
+					tprintf("= %ld", tcp->u_rval);
 					break;
 				default:
 					fprintf(stderr,
@@ -542,7 +830,8 @@ struct tcb *tcp;
 		}
 		if (dtime) {
 			tv_sub(&tv, &tv, &tcp->etime);
-			tprintf(" <%ld.%06ld>", tv.tv_sec, tv.tv_usec);
+			tprintf(" <%ld.%06ld>",
+				(long) tv.tv_sec, (long) tv.tv_usec);
 		}
 		printtrailer(tcp);
 
@@ -556,30 +845,70 @@ struct tcb *tcp;
 	/* Entering system call */
 	tcp->scno = scno;
 #ifdef LINUX
-	tcp->u_nargs = sysent[tcp->scno].nargs;
-	for (i = 0; i < tcp->u_nargs; i++) {
-		if (upeek(pid, i*4, &tcp->u_arg[i]) < 0)
-			return -1;
+#ifdef ALPHA
+	{
+		int i;
+		tcp->u_nargs = sysent[tcp->scno].nargs;
+		for (i = 0; i < tcp->u_nargs; i++) {
+			if (upeek(pid, REG_A0+i, &tcp->u_arg[i]) < 0)
+				return -1;
+		}
 	}
+#else /* !ALPHA */
+	{
+		int i;
+		tcp->u_nargs = sysent[tcp->scno].nargs;
+		for (i = 0; i < tcp->u_nargs; i++) {
+			if (upeek(pid, i*4, &tcp->u_arg[i]) < 0)
+				return -1;
+		}
+	}
+#endif
 #endif /* LINUX */
 #ifdef SUNOS4
-	tcp->u_nargs = sysent[tcp->scno].nargs;
-	for (i = 0; i < tcp->u_nargs; i++) {
-		struct user *u;
+	{
+		int i;
+		tcp->u_nargs = sysent[tcp->scno].nargs;
+		for (i = 0; i < tcp->u_nargs; i++) {
+			struct user *u;
 
-		if (upeek(pid, uoff(u_arg[0]) + (i*sizeof(u->u_arg[0])),
-		    &tcp->u_arg[i]) < 0)
-			return -1;
+			if (upeek(pid, uoff(u_arg[0]) +
+			    (i*sizeof(u->u_arg[0])), &tcp->u_arg[i]) < 0)
+				return -1;
+		}
 	}
 #endif /* SUNOS4 */
 #ifdef SVR4
+#ifdef MIPS
+	/*
+	 * SGI is broken: even though it has pr_sysarg, it doesn't
+	 * set them on system call entry.  Get a clue.
+	 */
+	if (sysent[tcp->scno].nargs != -1)
+		tcp->u_nargs = sysent[tcp->scno].nargs;
+	else
+		tcp->u_nargs = tcp->status.pr_nsysarg;
+	if (tcp->u_nargs > 4) {
+		memcpy(tcp->u_arg, &tcp->status.pr_reg[CTX_A0],
+			4*sizeof(tcp->u_arg[0]));
+		umoven(tcp, tcp->status.pr_reg[CTX_SP] + 16,
+			(tcp->u_nargs - 4)*sizeof(tcp->u_arg[0]), (char *) (tcp->u_arg + 4));
+	}
+	else {
+		memcpy(tcp->u_arg, &tcp->status.pr_reg[CTX_A0],
+			tcp->u_nargs*sizeof(tcp->u_arg[0]));
+	}
+#else /* !MIPS */
 #ifdef HAVE_PR_SYSCALL
 	if (sysent[tcp->scno].nargs != -1)
 		tcp->u_nargs = sysent[tcp->scno].nargs;
 	else
 		tcp->u_nargs = tcp->status.pr_nsysarg;
-	for (i = 0; i < tcp->u_nargs; i++)
-		tcp->u_arg[i] = tcp->status.pr_sysarg[i];
+	{
+		int i;
+		for (i = 0; i < tcp->u_nargs; i++)
+			tcp->u_arg[i] = tcp->status.pr_sysarg[i];
+	}
 #else /* !HAVE_PR_SYSCALL */
 #ifdef I386
 	if (sysent[tcp->scno].nargs != -1)
@@ -590,9 +919,11 @@ struct tcb *tcp;
 		tcp->u_nargs*sizeof(tcp->u_arg[0]), (char *) tcp->u_arg);
 #endif /* I386 */
 #endif /* !HAVE_PR_SYSCALL */
+#endif /* !MIPS */
 #endif /* SVR4 */
 	switch (tcp->scno) {
 #ifdef LINUX
+#ifndef ALPHA
 	case SYS_socketcall:
 		decode_subcall(tcp, SYS_socket_subcall,
 			SYS_socket_nsubcalls, deref_style);
@@ -601,16 +932,21 @@ struct tcb *tcp;
 		decode_subcall(tcp, SYS_ipc_subcall,
 			SYS_ipc_nsubcalls, shift_style);
 		break;
+#endif /* ALPHA */
 #endif /* LINUX */
 #ifdef SVR4
+#ifdef SYS_pgrpsys_subcall
 	case SYS_pgrpsys:
 		decode_subcall(tcp, SYS_pgrpsys_subcall,
 			SYS_pgrpsys_nsubcalls, shift_style);
 		break;
+#endif /* SYS_pgrpsys_subcall */
+#ifdef SYS_sigcall_subcall
 	case SYS_sigcall:
 		decode_subcall(tcp, SYS_sigcall_subcall,
 			SYS_sigcall_nsubcalls, mask_style);
 		break;
+#endif /* SYS_sigcall_subcall */
 	case SYS_msgsys:
 		decode_subcall(tcp, SYS_msgsys_subcall,
 			SYS_msgsys_nsubcalls, shift_style);
@@ -637,26 +973,46 @@ struct tcb *tcp;
 		decode_subcall(tcp, SYS_spcall_subcall,
 			SYS_spcall_nsubcalls, shift_style);
 		break;
+#ifdef SYS_context_subcall
 	case SYS_context:
 		decode_subcall(tcp, SYS_context_subcall,
 			SYS_context_nsubcalls, shift_style);
 		break;
+#endif /* SYS_context_subcall */
 #endif /* SVR4 */
+#ifdef SUNOS4
+	case SYS_semsys:
+		decode_subcall(tcp, SYS_semsys_subcall,
+			SYS_semsys_nsubcalls, shift_style);
+		break;
+	case SYS_msgsys:
+		decode_subcall(tcp, SYS_msgsys_subcall,
+			SYS_msgsys_nsubcalls, shift_style);
+		break;
+	case SYS_shmsys:
+		decode_subcall(tcp, SYS_shmsys_subcall,
+			SYS_shmsys_nsubcalls, shift_style);
+		break;
+#endif
 	}
+
+	internal_syscall(tcp);
 	if (!(qual_flags[tcp->scno] & QUAL_TRACE)) {
 		tcp->flags |= TCB_INSYSCALL;
 		return 0;
 	}
+
 	if (cflag) {
 		gettimeofday(&tcp->etime, NULL);
 		tcp->flags |= TCB_INSYSCALL;
 		return 0;
 	}
+
 	printleader(tcp);
 	tcp->flags &= ~TCB_REPRINT;
 	tcp_last = tcp;
 	if (tcp->scno >= nsyscalls)
-		tprintf("syscall_%u(", tcp->scno);
+		tprintf("syscall_%lu(", tcp->scno);
 	else
 		tprintf("%s(", sysent[tcp->scno].sys_name);
 	if (tcp->scno >= nsyscalls ||
@@ -681,16 +1037,16 @@ struct tcb *tcp;
 		int i;
 
 		for (i = 0; i < tcp->u_nargs; i++)
-			tprintf("%s%#x", i ? ", " : "", tcp->u_arg[i]);
+			tprintf("%s%#lx", i ? ", " : "", tcp->u_arg[i]);
 	}
 	return 0;
 }
 
-int
+long
 getrval2(tcp)
 struct tcb *tcp;
 {
-	int val = -1;
+	long val = -1;
 
 #ifdef SUNOS4
 	if (upeek(tcp->pid, uoff(u_rval2), &val) < 0)
@@ -704,6 +1060,9 @@ struct tcb *tcp;
 #ifdef I386
 	val = tcp->status.pr_reg[EDX];
 #endif /* I386 */
+#ifdef MIPS
+	val = tcp->status.pr_reg[CTX_V1];
+#endif /* MIPS */
 #endif /* SVR4 */
 
 	return val;
@@ -717,7 +1076,7 @@ int
 sys_indir(tcp)
 struct tcb *tcp;
 {
-	unsigned int i, scno, nargs;
+	int i, scno, nargs;
 
 	if (entering(tcp)) {
 		if ((scno = tcp->u_arg[0]) > nsyscalls) {
@@ -727,7 +1086,7 @@ struct tcb *tcp;
 		nargs = sysent[scno].nargs;
 		tprintf("%s", sysent[scno].sys_name);
 		for (i = 0; i < nargs; i++)
-			tprintf(", %#x", tcp->u_arg[i+1]);
+			tprintf(", %#lx", tcp->u_arg[i+1]);
 	}
 	return 0;
 }
@@ -831,9 +1190,10 @@ FILE *outf;
 			error_str[0] = '\0';
 		percent = 100.0*tv_float(&tv_count[j])/tv_float(&tv_cum);
 		fprintf(outf, "%6.2f %4ld.%06ld %11ld %9d %9.9s %s\n",
-			percent, tv_count[j].tv_sec, tv_count[j].tv_usec,
-			1000000 * dtv.tv_sec + dtv.tv_usec, call_count[j],
-			error_str, sysent[j].sys_name);
+			percent, (long) tv_count[j].tv_sec,
+			(long) tv_count[j].tv_usec,
+			(long) 1000000 * dtv.tv_sec + dtv.tv_usec,
+			call_count[j], error_str, sysent[j].sys_name);
 	}
 	fprintf(outf, "%6.6s %11.11s %11.11s %9.9s %9.9s %-16.16s\n",
 		dashes, dashes, dashes, dashes, dashes, dashes);
@@ -842,7 +1202,6 @@ FILE *outf;
 	else
 		error_str[0] = '\0';
 	fprintf(outf, "%6.6s %4ld.%06ld %11.11s %9d %9.9s %s\n",
-		"100.00", tv_cum.tv_sec, tv_cum.tv_usec, "", call_cum,
-		error_str, "total");
+		"100.00", (long) tv_cum.tv_sec, (long) tv_cum.tv_usec, "",
+		call_cum, error_str, "total");
 }
-
