@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 1991, 1992 Paul Kranenburg <pk@cs.few.eur.nl>
  * Copyright (c) 1993 Branko Lankester <branko@hacktic.nl>
- * Copyright (c) 1993, 1994 Rick Sladkey <jrs@world.std.com>
+ * Copyright (c) 1993, 1994, 1995, 1996 Rick Sladkey <jrs@world.std.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -12,11 +12,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by Paul Kranenburg,
- *      Branko Lankester and Rick Sladkey.
- * 4. The name of the author may not be used to endorse or promote products
+ * 3. The name of the author may not be used to endorse or promote products
  *    derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
@@ -30,7 +26,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	util.c,v 2.24 1994/03/21 05:45:46 jrs Exp
+ *	$Id: util.c,v 2.37 1996/05/21 03:32:23 jrs Exp $
  */
 
 #include "defs.h"
@@ -162,7 +158,7 @@ char *dflt;
 	if (str)
 		tprintf("%s", str);
 	else
-		tprintf("%s (%#x)", dflt, val);
+		tprintf("%#x /* %s */", val, dflt);
 }
 
 /*
@@ -223,7 +219,7 @@ int flags;
 void
 printnum(tcp, addr, fmt)
 struct tcb *tcp;
-int addr;
+long addr;
 char *fmt;
 {
 	int num;
@@ -233,7 +229,7 @@ char *fmt;
 		return;
 	}
 	if (umove(tcp, addr, &num) < 0) {
-		tprintf("%#x", addr);
+		tprintf("%#lx", addr);
 		return;
 	}
 	tprintf("[");
@@ -251,10 +247,10 @@ char *str;
 	char *s;
 
 	if (!strpbrk(str, "\"\'\\")) {
-		tprintf("\"%s\"", path);
+		tprintf("\"%s\"", str);
 		return;
 	}
-	for (s = buf; str; str++) {
+	for (s = buf; *str; str++) {
 		switch (*str) {
 		case '\"': case '\'': case '\\':
 			*s++ = '\\'; *s++ = *str; break;
@@ -269,10 +265,10 @@ char *str;
 void
 printpath(tcp, addr)
 struct tcb *tcp;
-int addr;
+long addr;
 {
 	if (umovestr(tcp, addr, MAXPATHLEN, path) < 0)
-		tprintf("%#x", addr);
+		tprintf("%#lx", addr);
 	else
 		string_quote(path);
 	return;
@@ -281,11 +277,11 @@ int addr;
 void
 printpathn(tcp, addr, n)
 struct tcb *tcp;
-int addr;
+long addr;
 int n;
 {
 	if (umovestr(tcp, addr, n, path) < 0)
-		tprintf("%#x", addr);
+		tprintf("%#lx", addr);
 	else {
 		path[n] = '\0';
 		string_quote(path);
@@ -295,7 +291,7 @@ int n;
 void
 printstr(tcp, addr, len)
 struct tcb *tcp;
-int addr;
+long addr;
 int len;
 {
 	static unsigned char *str = NULL;
@@ -311,22 +307,22 @@ int len;
 		if ((str = malloc(max_strlen)) == NULL
 		    || (outstr = malloc(2*max_strlen)) == NULL) {
 			fprintf(stderr, "printstr: no memory\n");
-			tprintf("%#x", addr);
+			tprintf("%#lx", addr);
 			return;
 		}
 	}
 	outend = outstr + max_strlen;
 	if (len < 0) {
 		n = max_strlen;
-		if (umovestr(tcp, addr, n, str) < 0) {
-			tprintf("%#x", addr);
+		if (umovestr(tcp, addr, n, (char *) str) < 0) {
+			tprintf("%#lx", addr);
 			return;
 		}
 	}
 	else {
 		n = MIN(len, max_strlen);
-		if (umoven(tcp, addr, n, str) < 0) {
-			tprintf("%#x", addr);
+		if (umoven(tcp, addr, n, (char *) str) < 0) {
+			tprintf("%#lx", addr);
 			return;
 		}
 	}
@@ -397,7 +393,7 @@ int len;
 	}
 
 	*s++ = '\"';
-	if (i < len || (len < 0 && i == n)) {
+	if (i < len || (len < 0 && (i == n || s > outend))) {
 		*s++ = '.'; *s++ = '.'; *s++ = '.';
 	}
 	*s = '\0';
@@ -407,7 +403,7 @@ int len;
 void
 dumpstr(tcp, addr, len)
 struct tcb *tcp;
-int addr;
+long addr;
 int len;
 {
 	static int strsize = -1;
@@ -426,7 +422,7 @@ int len;
 		strsize = len;
 	}
 
-	if (umoven(tcp, addr, len, str) < 0)
+	if (umoven(tcp, addr, len, (char *) str) < 0)
 		return;
 
 	for (i = 0; i < len; i += 16) {
@@ -436,7 +432,7 @@ int len;
 		for (j = 0; j < 16; j++) {
 			if (j == 8)
 				*s++ = ' ';
-			if (i + j <= len) {
+			if (i + j < len) {
 				sprintf(s, " %02x", str[i + j]);
 				s += 3;
 			}
@@ -448,7 +444,7 @@ int len;
 		for (j = 0; j < 16; j++) {
 			if (j == 8)
 				*s++ = ' ';
-			if (i + j <= len) {
+			if (i + j < len) {
 				if (isprint(str[i + j]))
 					*s++ = str[i + j];
 				else
@@ -469,11 +465,15 @@ int len;
 int
 umoven(tcp, addr, len, laddr)
 struct tcb *tcp;
-int addr, len;
+long addr;
+int len;
 char *laddr;
 {
 
 #ifdef LINUX
+#if 0
+	if (tcp->pfd == -2)
+		return -1;
 	if (tcp->pfd == -1) {
 		char buf[32];
 
@@ -481,64 +481,77 @@ char *laddr;
 		if ((tcp->pfd = open(buf, O_RDONLY)) == -1) {
 			perror(buf);
 			tcp->pfd = -2;
-		}
-	}
-	if (tcp->pfd != -2) {
-		lseek(tcp->pfd, addr, 0);
-		if (read(tcp->pfd, laddr, len) == -1) {
-			/* read somehow failed, switch to alternative method */
-			tcp->pfd = -2;
 			return -1;
 		}
 	}
-	if (tcp->pfd == -2) {
-		/* WTA: proc-filesystem failed, so we resort to ptrace.
-		 * copied from the SunOS oldway code. 
-		 */
-		int pid = tcp->pid;
-		int n, m;
-		union {
-			int val;
-			char x[4];
-		} u;
-	
-		if (addr & 3) {
-			/* addr not a multiple of 4 */
-			n = addr - (addr & -4); /* residue */
-			addr &= -4; /* residue */
-			u.val = ptrace(PTRACE_PEEKDATA, pid, (char *) addr, 0);
-			memcpy(laddr, &u.x[n], m = MIN(4 - n, len));
-			addr += 4, laddr += m, len -= m;
-		}
-		while (len) {
-			u.val = ptrace(PTRACE_PEEKDATA, pid, (char *) addr, 0);
-			memcpy(laddr, u.x, m = MIN(4, len));
-			addr += 4, laddr += m, len -= m;
+	lseek(tcp->pfd, addr, 0);
+	if (read(tcp->pfd, laddr, len) == -1)
+		return -1;
+#else
+	int pid = tcp->pid;
+	int n, m;
+	union {
+		long val;
+		char x[sizeof(long)];
+	} u;
+
+	if (addr & (sizeof(long) - 1)) {
+		/* addr not a multiple of sizeof(long) */
+		n = addr - (addr & -sizeof(long)); /* residue */
+		addr &= -sizeof(long); /* residue */
+		errno = 0;
+		u.val = ptrace(PTRACE_PEEKDATA, pid, (char *) addr, 0);
+		if (errno) {
+			perror("ptrace: umoven");
+			return -1;
 		}
+		memcpy(laddr, &u.x[n], m = MIN(sizeof(long) - n, len));
+		addr += sizeof(long), laddr += m, len -= m;
 	}
+	while (len) {
+		errno = 0;
+		u.val = ptrace(PTRACE_PEEKDATA, pid, (char *) addr, 0);
+		if (errno) {
+			perror("ptrace: umoven");
+			return -1;
+		}
+		memcpy(laddr, u.x, m = MIN(sizeof(long), len));
+		addr += sizeof(long), laddr += m, len -= m;
+	}
+#endif
 #endif /* LINUX */
 
 #ifdef SUNOS4
 	int pid = tcp->pid;
-#ifdef oldway
+#if 0
 	int n, m;
 	union {
-		int val;
-		char x[4];
+		long val;
+		char x[sizeof(long)];
 	} u;
 
-	if (addr & 3) {
-		/* addr not a multiple of 4 */
-		n = addr - (addr & -4); /* residue */
-		addr &= -4; /* residue */
+	if (addr & (sizeof(long) - 1)) {
+		/* addr not a multiple of sizeof(long) */
+		n = addr - (addr & -sizeof(long)); /* residue */
+		addr &= -sizeof(long); /* residue */
+		errno = 0;
 		u.val = ptrace(PTRACE_PEEKDATA, pid, (char *) addr, 0);
-		memcpy(laddr, &u.x[n], m = MIN(4 - n, len));
-		addr += 4, laddr += m, len -= m;
+		if (errno) {
+			perror("umoven");
+			return -1;
+		}
+		memcpy(laddr, &u.x[n], m = MIN(sizeof(long) - n, len));
+		addr += sizeof(long), laddr += m, len -= m;
 	}
 	while (len) {
+		errno = 0;
 		u.val = ptrace(PTRACE_PEEKDATA, pid, (char *) addr, 0);
-		memcpy(laddr, u.x, m = MIN(4, len));
-		addr += 4, laddr += m, len -= m;
+		if (errno) {
+			perror("umoven");
+			return -1;
+		}
+		memcpy(laddr, u.x, m = MIN(sizeof(long), len));
+		addr += sizeof(long), laddr += m, len -= m;
 	}
 #else /* !oldway */
 	int n;
@@ -560,6 +573,13 @@ char *laddr;
 #endif /* SUNOS4 */
 
 #ifdef SVR4
+/*
+ * We would like to use pread preferentially for speed
+ * but even though SGI has it in their library, it no longer works.
+ */
+#ifdef MIPS
+#undef HAVE_PREAD
+#endif
 #ifdef HAVE_PREAD
 	if (pread(tcp->pfd, laddr, len, addr) == -1)
 		return -1;
@@ -580,7 +600,8 @@ char *laddr;
 int
 umovestr(tcp, addr, len, laddr)
 struct tcb *tcp;
-int addr, len;
+long addr;
+int len;
 char *laddr;
 {
 #ifndef SUNOS4
@@ -589,24 +610,25 @@ char *laddr;
 	int pid = tcp->pid;
 	int i, n, m;
 	union {
-		int val;
-		char x[4];
+		long val;
+		char x[sizeof(long)];
 	} u;
 
-	if (addr & 3) /* addr not a multiple of 4 */ {
-		n = addr - (addr & -4); /* residue */
-		addr &= -4; /* residue */
+	if (addr & (sizeof(long) - 1)) {
+		/* addr not a multiple of sizeof(long) */
+		n = addr - (addr & -sizeof(long)); /* residue */
+		addr &= -sizeof(long); /* residue */
 		errno = 0;
 		u.val = ptrace(PTRACE_PEEKDATA, pid, (char *)addr, 0);
 		if (errno) {
 			perror("umovestr");
 			return -1;
 		}
-		memcpy(laddr, &u.x[n], m = MIN(4-n,len));
-		while (n & 3)
+		memcpy(laddr, &u.x[n], m = MIN(sizeof(long)-n,len));
+		while (n & (sizeof(long) - 1))
 			if (u.x[n++] == '\0')
 				return 0;
-		addr += 4, laddr += m, len -= m;
+		addr += sizeof(long), laddr += m, len -= m;
 	}
 	while (len) {
 		errno = 0;
@@ -615,12 +637,12 @@ char *laddr;
 			perror("umovestr");
 			return -1;
 		}
-		memcpy(laddr, u.x, m = MIN(4, len));
-		for (i = 0; i < 4; i++)
+		memcpy(laddr, u.x, m = MIN(sizeof(long), len));
+		for (i = 0; i < sizeof(long); i++)
 			if (u.x[i] == '\0')
 				return 0;
 
-		addr += 4, laddr += m, len -= m;
+		addr += sizeof(long), laddr += m, len -= m;
 	}
 	return 0;
 #endif /* SUNOS4 */
@@ -631,13 +653,14 @@ char *laddr;
 #define PTRACE_WRITEDATA	102
 #endif /* LINUX */
 
-#ifndef SVR4
+#ifdef SUNOS4
 
 static int
 uload(cmd, pid, addr, len, laddr)
 int cmd;
 int pid;
-int addr, len;
+long addr;
+int len;
 char *laddr;
 {
 #if 0
@@ -658,8 +681,8 @@ char *laddr;
 	int peek, poke;
 	int n, m;
 	union {
-		int val;
-		char x[4];
+		long val;
+		char x[sizeof(long)];
 	} u;
 
 	if (cmd == PTRACE_WRITETEXT) {
@@ -670,27 +693,32 @@ char *laddr;
 		peek = PTRACE_PEEKDATA;
 		poke = PTRACE_POKEDATA;
 	}
-	if (addr & 3) {
-		/* addr not a multiple of 4 */
-		n = addr - (addr & -4); /* residue */
-		addr &= -4;
+	if (addr & (sizeof(long) - 1)) {
+		/* addr not a multiple of sizeof(long) */
+		n = addr - (addr & -sizeof(long)); /* residue */
+		addr &= -sizeof(long);
+		errno = 0;
 		u.val = ptrace(peek, pid, (char *) addr, 0);
-		memcpy(&u.x[n], laddr, m = MIN(4 - n, len));
+		if (errno) {
+			perror("uload: POKE");
+			return -1;
+		}
+		memcpy(&u.x[n], laddr, m = MIN(sizeof(long) - n, len));
 		if (ptrace(poke, pid, (char *)addr, u.val) < 0) {
 			perror("uload: POKE");
 			return -1;
 		}
-		addr += 4, laddr += m, len -= m;
+		addr += sizeof(long), laddr += m, len -= m;
 	}
 	while (len) {
-		if (len < 4)
+		if (len < sizeof(long))
 			u.val = ptrace(peek, pid, (char *) addr, 0);
-		memcpy(u.x, laddr, m = MIN(4, len));
+		memcpy(u.x, laddr, m = MIN(sizeof(long), len));
 		if (ptrace(poke, pid, (char *) addr, u.val) < 0) {
 			perror("uload: POKE");
 			return -1;
 		}
-		addr += 4, laddr += m, len -= m;
+		addr += sizeof(long), laddr += m, len -= m;
 	}
 #endif
 	return 0;
@@ -715,13 +743,17 @@ char *laddr;
 	return uload(PTRACE_WRITEDATA, pid, addr, len, laddr);
 }
 
+#endif /* SUNOS4 */
+
+#ifndef SVR4
+
 int
 upeek(pid, off, res)
 int pid;
-int off;
-int *res;
+long off;
+long *res;
 {
-	int val;
+	long val;
 
 #ifdef SUNOS4_KERNEL_ARCH_KLUDGE
 	{
@@ -748,7 +780,7 @@ int *res;
 	}
 #endif /* SUNOS4_KERNEL_ARCH_KLUDGE */
 	errno = 0;
-	val = ptrace(PTRACE_PEEKUSER, pid, (char *)off, 0);
+	val = ptrace(PTRACE_PEEKUSER, pid, (char *) off, 0);
 	if (val == -1 && errno) {
 		perror("upeek: ptrace(PTRACE_PEEKUSER, ... )");
 		return -1;
@@ -759,16 +791,28 @@ int *res;
 
 #endif /* !SVR4 */
 
-int
+long
 getpc(tcp)
 struct tcb *tcp;
 {
 
 #ifdef LINUX
-	int pc;
+	long pc;
 
+#ifdef I386
 	if (upeek(tcp->pid, 4*EIP, &pc) < 0)
 		return -1;
+#else /* !I386 */
+#ifdef M68K
+	if (upeek(tcp->pid, 4*PT_PC, &pc) < 0)
+		return -1;
+#else /* !M68K */
+#ifdef ALPHA
+	if (upeek(tcp->pid, REG_PC, &pc) < 0)
+		return -1;
+#endif /* ALPHA */
+#endif /* !M68K */
+#endif /* !I386 */
 	return pc;
 #endif /* LINUX */
 
@@ -799,13 +843,35 @@ struct tcb *tcp;
 {
 
 #ifdef LINUX
-	int eip;
+#ifdef I386
+	long eip;
 
 	if (upeek(tcp->pid, 4*EIP, &eip) < 0) {
 		tprintf("[????????] ");
 		return;
 	}
-	tprintf("[%08x] ", eip);
+	tprintf("[%08lx] ", eip);
+#else /* !I386K */
+#ifdef M68K
+	long pc;
+
+	if (upeek(tcp->pid, 4*PT_PC, &pc) < 0) {
+		tprintf ("[????????] ");
+		return;
+	}
+	tprintf("[%08lx] ", pc);
+#else /* !M68K */
+#ifdef ALPHA
+	long pc;
+
+	if (upeek(tcp->pid, REG_PC, &pc) < 0) {
+		tprintf ("[????????] ");
+		return;
+	}
+	tprintf("[%08lx] ", pc);
+#endif /* ALPHA */
+#endif /* !M68K */
+#endif /* !I386 */
 #endif /* LINUX */
 
 #ifdef SUNOS4
@@ -835,16 +901,38 @@ struct tcb *tcp;
 
 #ifdef LINUX
 
+#ifdef I386
 #define LOOP	0x0000feeb
+#else /* !I386 */
+#ifdef M68K
+#define LOOP	0x60fe0000
+#else /* !M68K */
+#ifdef ALPHA
+#define LOOP    0xc3ffffff
+#endif /* ALPHA */
+#endif /* !M68K */
+#endif /* !I386 */
 
 	if (tcp->flags & TCB_BPTSET) {
 		fprintf(stderr, "PANIC: bpt already set in pid %u\n", tcp->pid);
 		return -1;
 	}
+#ifdef I386
 	if (upeek(tcp->pid, 4*EIP, &tcp->baddr) < 0)
 		return -1;
+#else /* !I386 */
+#ifdef M68K
+	if (upeek(tcp->pid, 4*PT_PC, &tcp->baddr) < 0)
+	  return -1;
+#else /* !M68K */
+#ifdef ALPHA
+	if (upeek(tcp->pid, REG_PC, &tcp->baddr) < 0)
+	  return -1;
+#endif /* ALPHA */
+#endif /* !M68K */
+#endif /* !I386 */
 	if (debug)
-		fprintf(stderr, "[%d] setting bpt at %x\n", tcp->pid, tcp->baddr);
+		fprintf(stderr, "[%d] setting bpt at %lx\n", tcp->pid, tcp->baddr);
 	tcp->inst[0] = ptrace(PTRACE_PEEKTEXT, tcp->pid, (char *) tcp->baddr, 0);
 	if (errno) {
 		perror("setbpt: ptrace(PTRACE_PEEKTEXT, ...)");
@@ -917,7 +1005,17 @@ struct tcb *tcp;
 {
 
 #ifdef LINUX
-	int eip;
+#ifdef I386
+	long eip;
+#else /* !I386 */
+#ifdef M68K
+	long pc;
+#else /* !M68K */
+#ifdef ALPHA
+	long pc;
+#endif /* ALPHA */
+#endif /* !M68K */
+#endif /* !I386 */
 
 	if (debug)
 		fprintf(stderr, "[%d] clearing bpt\n", tcp->pid);
@@ -933,16 +1031,42 @@ struct tcb *tcp;
 	}
 	tcp->flags &= ~TCB_BPTSET;
 
+#ifdef I386
 	if (upeek(tcp->pid, 4*EIP, &eip) < 0)
 		return -1;
 	if (eip != tcp->baddr) {
-		/* The breakpoint has not been reached yet */
+		/* The breakpoint has not been reached yet.  */
 		if (debug)
 			fprintf(stderr,
-				"NOTE: PC not at bpt (pc %#x baddr %#x)\n",
+				"NOTE: PC not at bpt (pc %#lx baddr %#lx)\n",
 					eip, tcp->baddr);
 		return 0;
 	}
+#else /* !I386 */
+#ifdef M68K
+	if (upeek(tcp->pid, 4*PT_PC, &pc) < 0)
+		return -1;
+	if (pc != tcp->baddr) {
+		/* The breakpoint has not been reached yet.  */
+		if (debug)
+			fprintf(stderr, "NOTE: PC not at bpt (pc %#lx baddr %#lx)\n",
+				pc, tcp->baddr);
+		return 0;
+	}
+#else /* !M68K */
+#ifdef ALPHA
+	if (upeek(tcp->pid, REG_PC, &pc) < 0)
+		return -1;
+	if (pc != tcp->baddr) {
+		/* The breakpoint has not been reached yet.  */
+		if (debug)
+			fprintf(stderr, "NOTE: PC not at bpt (pc %#lx baddr %#lx)\n",
+				pc, tcp->baddr);
+		return 0;
+	}
+#endif /* ALPHA */
+#endif /* !M68K */
+#endif /* !I386 */
 #endif /* LINUX */
 
 #ifdef SUNOS4
@@ -1010,7 +1134,7 @@ struct exec *hdr;
 	int n;
 
 	for (n = 0; n < sizeof *hdr; n += 4) {
-		int res;
+		long res;
 		if (upeek(pid, uoff(u_exdata) + n, &res) < 0)
 			return -1;
 		memcpy(((char *) hdr) + n, &res, 4);
